<?php

include_once '../objects/linia.php';
include_once '../objects/parada.php';
include_once '../objects/expedicio.php';

class Avis {
    //connexió a la base de dades
    private $conn;

     // propietats de l'objecte
     public $id;
     public $tipusObjecte;
     public $objecte;
     public $tipusAvis;
     public $titol;
     public $text;
     public $dataInici;
     public $dataFinal;


    //Constructor amb la connexió a la base de dades
    public function __construct($db){
        $this->conn = $db;
    }

        //FUNCIÓ OBTÉ DADES
        //Consulta a la base de dades la informació de l'objecte i l'assigna a les seves variables corresponents
        public function obteDades(){
            $query = "SELECT A.titol, A.text, A.dataInici, A.dataFinal, A.tipusObjecte, A.idObjecte, B.nom, B.id AS idTipusAvis FROM Avis A            
                        JOIN TipusAvis B ON A.tipusAvis = B.id
                    WHERE
                        A.id = ?
                   ";
            $stmt = $this->conn->prepare( $query );
            $stmt->bindParam(1, $this->id);
            $stmt->execute();

            $row = $stmt->fetch(PDO::FETCH_ASSOC);
            
            $this->tipusAvis=$row['idTipusAvis']; //Es nom perquè ve del join de TipusAvis B
            $this->titol=$row['titol'];
            $this->text=$row['text'];
            $this->dataInici = ($row['dataInici']!=null ? date("Y-m-d", strtotime($row['dataInici'])) : null);
            $this->dataFinal = ($row['dataFinal']!=null ? date("Y-m-d", strtotime($row['dataFinal'])) : null);
            $this->tipusObjecte=$row['tipusObjecte'];

            switch ($this->tipusObjecte) { //Segons el tipus d'objecte en creem un o altre
                case "parada":
                    $this->objecte = new Parada($this->conn);
                    $this->objecte ->id = $row['idObjecte'];
                    break;
                case "linia":
                    $this->objecte  = new Linia($this->conn);
                    $this->objecte ->codi = $row['idObjecte'];
                    break;
                case "expedicio":
                    $this->objecte  = new Expedicio($this->conn);
                    $this->objecte->id = $row['idObjecte'];
                    break;
                default: break;
            }

            !is_null($this->tipusObjecte) && $this->objecte->obteDades(); //Com que tots els objectes tenen el mateix mètode, el podem cridar des d'aquí
            
        }

        //FUNCIÓ OBTÉ DADES SENSE OBJECTE
        //Consulta a la base de dades la informació de l'objecte i l'assigna a les seves variables corresponents
        //A diferència de l'anterior funció, no retorna les dades de l'objecte
        public function obteDadesSenseObjecte(){
            $query = "SELECT A.titol, A.text, A.dataInici, A.dataFinal, A.tipusObjecte, A.idObjecte, B.nom, B.id AS idTipusAvis FROM Avis A            
                        JOIN TipusAvis B ON A.tipusAvis = B.id
                    WHERE
                        A.id = ?
                   ";
            $stmt = $this->conn->prepare( $query );
            $stmt->bindParam(1, $this->id);
            $stmt->execute();

            $row = $stmt->fetch(PDO::FETCH_ASSOC);
            
            $this->tipusAvis=$row['idTipusAvis'];
            $this->titol=$row['titol'];
            $this->text=$row['text'];
            $this->dataInici = ($row['dataInici']!=null ? date("Y-m-d", strtotime($row['dataInici'])) : null);
            $this->dataFinal = ($row['dataFinal']!=null ? date("Y-m-d", strtotime($row['dataFinal'])) : null);
            $this->tipusObjecte=null;
            
        }

        //FUNCIÓ OBTÉ AVISOS PER OBJECTE
        // retorna Array amb avisos d'un objecte concret
        public function obteAvisosPerObjecte($tipusObjecte, $idObjecte){
            $data=date("Y-m-d"); //Agafa data del dia d'avui, només carreguem els avisos que no han caducat (<dataFinal)            
            $query = "SELECT * FROM Avis A            
                    WHERE
                        A.idObjecte = :idObjecte
                    AND A.tipusObjecte = :tipusObjecte
                    AND (A.dataFinal>=:data OR A.dataFinal IS NULL)
                    ORDER BY A.dataFinal
                   ";
            $stmt = $this->conn->prepare( $query );
            $stmt->bindParam(":idObjecte", $idObjecte);
            $stmt->bindParam(":tipusObjecte", $tipusObjecte);
            $stmt->bindParam(":data", $data);
            $stmt->execute();
            $arrayAvisos=array();

            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                $avis = new Avis($this->conn); //Crea un objecte Avis per a cada resposta
                $avis->id = $row["id"];
                $avis->obteDades(); 
                array_push($arrayAvisos, $avis);
            }
            return $arrayAvisos;            
        }

        //FUNCIÓ OBTÉ TOTS AVISOS
        // retorna Array amb tots els avisos
        public function obteTotsAvisos(){ 
            $query = "SELECT * FROM Avis A            
                    ORDER BY A.dataFinal
                   ";
            $stmt = $this->conn->prepare( $query );
            $stmt->execute();
            $arrayAvisos=array();

            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                $avis = new Avis($this->conn); //Crea un objecte Avis per a cada resposta
                $avis->id = $row["id"];
                $avis->obteDades(); 
                array_push($arrayAvisos, $avis);
            }
            return $arrayAvisos;            
        }

        //FUNCIÓ OBTÉ TIPUS AVIS
        // retorna Array amb tots els avisos
        public function obteTipusAvisos(){ 
            $query = "SELECT * FROM TipusAvis A            
                    ORDER BY A.id
                   ";
            $stmt = $this->conn->prepare( $query );
            $stmt->execute();
            $arrayTipusAvis=array();

            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                array_push($arrayTipusAvis, $row);
            }
            return $arrayTipusAvis;            
        }

         //FUNCIÓ OBTÉ TOTS AVISOS SENSE EXPEDICIONS
        // retorna Array amb tots els avisos
        public function obteTotsAvisosSenseExpedicions(){
            $data=date("Y-m-d"); //Agafa data del dia d'avui, només carreguem els avisos que no han caducat (<dataFinal)            
            $query = "SELECT * FROM Avis A            
                    WHERE (A.dataFinal>=:data OR A.dataFinal IS NULL)
                    AND A.tipusObjecte!='expedicio'
                    ORDER BY A.dataFinal
                   ";
            $stmt = $this->conn->prepare( $query );
            $stmt->bindParam(":data", $data);
            $stmt->execute();
            $arrayAvisos=array();

            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                $avis = new Avis($this->conn); //Crea un objecte Avis per a cada resposta
                $avis->id = $row["id"];
                $avis->obteDades(); 
                array_push($arrayAvisos, $avis);
            }
            return $arrayAvisos;            
        }

            //FUNCIO AFEGIR
            //Funció per afegir una nou Avis. Previament s'han d'haver assignat tots els valors a les variables de l'objecte
            public function afegir(){
                $query = "INSERT INTO Avis
                                SET ";
            $stmt = $this->conn->prepare( $query );

            echo $this->text;

            //Assignem els valors amagats de la petició
            $stmt->bindParam(":tipusObjecte", $this->tipusObjecte);
            $stmt->bindParam(":idObjecte", $this->objecte);
            $stmt->bindParam(":tipusAvis", $this->tipusAvis);
            $stmt->bindParam(":dataInici", $this->dataInici);
            $stmt->bindParam(":dataFinal", $this->dataFinal);
            $stmt->bindParam(":titol", $this->titol);
            $stmt->bindParam(":text", $this->text);
        
            //Executem la consulta, si va bé retorna true, si no, false.
            if($stmt->execute()){
                $this->id = $this->conn->lastInsertId(); //Obtenim l'id de l'avis introduit
                return true;
            }
         
            return false;    
        
            }


         //FUNCIO MODIFICAR
        //Funció per modificar un avis. Previament s'han d'haver assignat tots els valors a les variables de l'objecte
        public function modificar(){
            $query = "UPDATE Avis
                            SET tipusObjecte=:tipusObjecte, idObjecte=:idObjecte, tipusAvis=:tipusAvis, dataInici=:dataInici, dataFinal=:dataFinal, titol=:titol, text=:text
                            WHERE id = :idAvis";
        $stmt = $this->conn->prepare( $query );
    
        //Assignem els valors amagats de la petició
        $stmt->bindParam(":idAvis", $this->id);
        $stmt->bindParam(":tipusObjecte", $this->tipusObjecte);
        $stmt->bindParam(":idObjecte", $this->objecte);
        $stmt->bindParam(":tipusAvis", $this->tipusAvis);
        $stmt->bindParam(":dataInici", $this->dataInici);
        $stmt->bindParam(":dataFinal", $this->dataFinal);
        $stmt->bindParam(":titol", $this->titol);
        $stmt->bindParam(":text", $this->text);
    
        //Executem la consulta, si va bé retorna true, si no, false.
        if($stmt->execute()){
            return true;
        }
     
        return false;    
    
        }

        //FUNCIO ELIMINAR
        //Funció per eliminar un Avis. Previament s'han d'haver assignat tots els valors a les variables de l'objecte
        public function eliminar(){
            $query = "DELETE FROM Avis WHERE id = :idAvis";
            $stmt = $this->conn->prepare( $query );
        
            //Assignem els valors amagats de la petició
            $stmt->bindParam(":idAvis", $this->id);
        
            //Executem la consulta, si va bé retorna true, si no, false.
            if($stmt->execute()){
                return true;
            }
         
            return false;    
        
        } 
}

?>