<?php
include_once '../objects/parada.php';
include_once '../objects/expedicio.php';


class AvisUsuari {

    //connexió a la base de dades
    private $conn;

    // propietats de l'objecte
    public $id;
    public $idExpedicio;
    public $expedicio;
    public $idParada;
    public $parada;
    public $tipusAvis;
    public $retard;
    public $hora;
    public $data;

    //Constructor amb la connexió a la base de dades
    public function __construct($db){
        $this->conn = $db;
    }


    //FUNCIÓ OBTÉ DADES
    //Consulta a la base de dades la informació de l'objecte i l'assigna a les seves variables corresponents
    public function obteDades(){
        $query = "SELECT * FROM AvisUsuari A
                WHERE
                    A.id = ?
               ";
        $stmt = $this->conn->prepare( $query );
    
        // assignarà el valor de id que rebrà a l'interrogant que hem deixat a la consulta.
        $stmt->bindParam(1, $this->id);
        $stmt->execute();
    
        //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
        $row = $stmt->fetch(PDO::FETCH_ASSOC);
        
        $this->idExpedicio = $row['idExpedicio'];
        $this->expedicio = new Expedicio($this->conn);
        $this->expedicio->id = $this->idExpedicio;
        $this->expedicio->obteDades();

        $this->idParada = $row['idParada'];
        $this->parada = new Parada($this->conn);
        $this->parada->id = $this->idParada;
        $this->parada->obteDades();
        
        $this->tipusAvis = $row['tipusAvis'];
        $this->retard = $row['retard'];
        $this->hora = $row['dataAvis'];
        $this->hora = date("H:i", strtotime($this->hora));

        $this->data = $row['dataAvis'];
        $this->data = date("d-m-Y", strtotime($this->data));
        
    }

    //FUNCIÓ AFEGIR
    //Afegeix a la base de dades l'avis. Cal tenir assignades les variables
    public function afegir(){
        $query = "INSERT INTO AvisUsuari
                        SET idExpedicio=:idExpedicio, idParada=:idParada, tipusAvis=:tipusAvis, retard=:retard";
    $stmt = $this->conn->prepare( $query );

    //Assignem els valors amagats de la petició
    $stmt->bindParam(":idExpedicio", $this->idExpedicio);
    $stmt->bindParam(":idParada", $this->idParada);
    $stmt->bindParam(":tipusAvis", $this->tipusAvis);
    $stmt->bindParam(":retard", $this->retard);

    //Executem la consulta, si va bé retorna true, si no, false.
    if($stmt->execute()){
        return true;
    }
 
    return false;    

    }

    //FUNCIÓ OBTE AVISOS EXPEDICIO
     // carrega informació d'un avis concret
     public function obteAvisosExpedicio(){
        $query = "SELECT * , DATE_FORMAT(A.dataAvis, '%Y-%m-%d') FROM AvisUsuari A 
                WHERE
                    A.idExpedicio = :idExpedicio
                    AND DATE(dataAvis) = CURDATE()
               "; //Fem que només carreguin els avisos introduits durant el dia d'avui
        $stmt = $this->conn->prepare( $query );
        
        //Assignem els valors amagats de la petició
        $stmt->bindParam(":idExpedicio", $this->idExpedicio);

        $stmt->execute();

        $arrayAvisos=array();
       
            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                $avisUsuari = new AvisUsuari($this->conn); //Crea un objecte Línia per a cada resposta
                $avisUsuari->id= $row["id"];
                $avisUsuari->obteDades(); 
                array_push($arrayAvisos, $avisUsuari);
            }

            return $arrayAvisos;
        
    }

        //FUNCIÓ OBTE AVISOS EXPEDICIO
     // carrega informació d'un avis concret
     public function obteTots(){
        $query = "SELECT * , DATE_FORMAT(A.dataAvis, '%Y-%m-%d') FROM AvisUsuari A ORDER BY A.id DESC
               "; //Fem que només carreguin els avisos introduits durant el dia d'avui
        $stmt = $this->conn->prepare( $query );
        
        $stmt->execute();

        $arrayAvisos=array();
       
            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                $avisUsuari = new AvisUsuari($this->conn); //Crea un objecte Línia per a cada resposta
                $avisUsuari->id= $row["id"];
                $avisUsuari->obteDades(); 
                array_push($arrayAvisos, $avisUsuari);
            }

            return $arrayAvisos;
        
    }



}

?>