<?php

include_once '../objects/linia.php';
include_once '../objects/parada.php';
include_once '../objects/avis.php';
include_once '../objects/avisUsuari.php';

include_once '../funcions/polyline.php';
include_once '../funcions/funcions.php';

class Expedicio {

    //connexió a la base de dades
    private $conn;

    // propietats de l'objecte
    public $id;
    public $linia;
    public $invers;
    public $paradaInici;
    public $paradaFinal;
    public $diesCirculacio; //0 és diumenge
    public $horari;
    public $diaInici;
    public $diaFinal;

     //Constructor amb la connexió a la base de dades
     public function __construct($db){
        $this->conn = $db;
    }

        //FUNCIÓ OBTÉ DADES
        //Consulta a la base de dades la informació de l'objecte i l'assigna a les seves variables corresponents
        public function obteDades(){
            $query = "SELECT * FROM Expedicio A
                    WHERE
                        A.id = ?
                   ";
            $stmt = $this->conn->prepare( $query );
        
            // assignarà el valor de id que rebrà a l'interrogant que hem deixat a la consulta.
            $stmt->bindParam(1, $this->id);
            $stmt->execute();
        
            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            $row = $stmt->fetch(PDO::FETCH_ASSOC);
            
            $this->linia = new Linia($this->conn);
            $this->linia->codi = $row['codiLinia'];
            $this->linia->obteDades(); 
            
            $this->paradaInici = new Parada($this->conn);
            $this->paradaInici->id = $row['paradaInici'];
            $this->paradaInici->obteDades();
            $this->paradaFinal = new Parada($this->conn);
            $this->paradaFinal->id = $row['paradaFinal'];
            $this->paradaFinal->obteDades();

            $this->diesCirculacio=array($row['festiu'],$row['dilluns'],$row['dimarts'],$row['dimecres'],$row['dijous'],$row['divendres'],$row['dissabte']);

            $this->diaInici = ($row['diaInici']!=null ? date("Y-m-d", strtotime($row['diaInici'])) : null);
            $this->diaFinal = ($row['diaFinal']!=null ? date("Y-m-d", strtotime($row['diaFinal'])) : null);

            //Calculem invers segons l'ordre de la línia
            if ($this->paradaInici->obteOrdre($this->linia)<$this->paradaFinal->obteOrdre($this->linia)){
                $this->invers="DESC";
            } else  $this->invers="ASC";
            
        }

        //FUNCIO ARRAY HORA PAS
        //Retorna un array amb les parades i la seva hora de pas de l'expedició
        public function arrayHoraPas() {
            $query = "SELECT * FROM Horari A
                    WHERE
                        A.idExpedicio = ?
                        ORDER BY A.horaPas
                   ";
            $stmt = $this->conn->prepare( $query );
        
            // assignarà el valor de id que rebrà a l'interrogant que hem deixat a la consulta.
            $stmt->bindParam(1, $this->id);
            $stmt->execute();
            $arrayParades=array();
            while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                $parada= new Parada($this->conn); //Crea un objecte Línia per a cada resposta
                $parada->id = $row["idParada"];
                $parada->obteDades();
                $horaPas = date("H:i", strtotime($row["horaPas"]));

                $arrayParada=array($parada, $horaPas);
                array_push($arrayParades, $arrayParada);
            }
            return $arrayParades;

        }

                    // funció OBTE TOTES
            // retorna una Array amb TOTES les expedicions de la BD o només les de la línia concreta si hi ha assignada una línia a l'objecte
            public function obteTotes(){
                
                if(empty($this->linia)) { //Si no tenim la dada de la línia retorna tots els arrays, si no, només els de la linia.
                    $query = "SELECT A.id FROM Expedicio A JOIN Horari B ON A.id=B.idExpedicio AND A.paradaInici = B.idParada ORDER BY B.horaPas ASC";
                } else $query = "SELECT A.id FROM Expedicio A JOIN Horari B ON A.id=B.idExpedicio AND A.paradaInici = B.idParada WHERE A.codiLinia=:codiLinia ORDER BY B.horaPas ASC";

                $stmt = $this->conn->prepare($query);
                (!empty($this->linia)) && $stmt->bindParam("codiLinia", $this->linia->codi);

                $stmt->execute();
                $arrayExpedicions=array();

                //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
                while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                    $expedicio = new Expedicio($this->conn); //Crea un objecte Parada per a cada resposta
                    $expedicio->id = $row["id"];
                    $expedicio->obteDades(); 
                    array_push($arrayExpedicions, $expedicio);
                }
                return $arrayExpedicions;
            }

        //FUNCIÓ OBTÉ PARADES
        //Troba les parades que formen part de l'expedició
        public function obteParades(){
            $query = "SELECT A.idParada FROM Horari A
                        WHERE A.idExpedicio = ?
                        ORDER BY A.horaPas
                   ";
            $stmt = $this->conn->prepare( $query );
        
            // assignarà el valor de id que rebrà a l'interrogant que hem deixat a la consulta.
            $stmt->bindParam(1, $this->id);
            $stmt->execute();
            $arrayParades=array();

            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                $parada = new Parada($this->conn); //Crea un objecte Línia per a cada resposta
                $parada->id = $row["idParada"];
                $parada->obteDades(); 
                array_push($arrayParades, $parada);
            }
            return $arrayParades;            
        }
            //FUNCIÓ OBTÉ PARADES EN RECORREGUT
            //FUNCIÓ PER OBTENIR LES PARADES INTERMEDIES D'UN RECORREGUT CONCRET
    public function obteParadesEnRecorregut($horaPasOrigen, $horaPasDestinacio){
        $query = "SELECT idParada FROM Horari A
                       WHERE A.idExpedicio = ?
                       AND A.horaPas>= ?
                       AND A.horaPas<= ? ;
              ";
       
       $stmt = $this->conn->prepare( $query );
          // assignarà el valor de id que rebrà a l'interrogant que hem deixat a la consulta.
       $stmt->bindParam(1, $this->id);
       $stmt->bindParam(2, $horaPasOrigen);
       $stmt->bindParam(3, $horaPasDestinacio);
       $stmt->execute();
       $arrayParades=array();

       //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
       while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
           $parada = new Parada($this->conn); //Crea un objecte Parada per a cada resposta
           $parada->id = $row["idParada"];
           $parada->obteDades(); 
           array_push($arrayParades, $parada);
       }
       return $arrayParades;
       
   }

        //FUNCIÓ TROBA HORA PAS
        //Retorna l'hora de pas de la parada que passem com a atribut si aquesta forma part de l'expedició
        public function trobaHoraPas($parada) {
            $query = "SELECT * FROM Horari A
                    WHERE
                        A.idExpedicio = ?
                    AND
                        A.idParada = ?
                   ";
            $stmt = $this->conn->prepare( $query );
        
            // assignarà el valor de id que rebrà a l'interrogant que hem deixat a la consulta.
            $stmt->bindParam(1, $this->id);
            $stmt->bindParam(2, $parada->id);
            $stmt->execute();
        
            //Si existeix l'hora de pas, s'assigna a la variable que es retornara
            $resposta = $stmt->fetch(PDO::FETCH_ASSOC);
            if (!is_null($resposta["horaPas"])){ 
                $horaPas = date("H:i", strtotime($resposta["horaPas"]));
            } else $horaPas=null;
            
            return $horaPas;

        }

        //FUNCIÓ OBTÉ POLILINIA
        //Funció que retorna una polilinia codificada per Google Maps        
        public function obtePolilinia(){ //Funció que retorna una polilinia codificada per Google Maps
            $parades=$this->obteParades();
            $arrayParades = array();
            if (count($parades)>0) {
                $i=0;
                foreach ($parades as $parada) {
                            $arrayParada = array (floatval($parada->latitud), floatval($parada->longitud));
                            array_push($arrayParades, $arrayParada);
                            $i++;                
                }
            }
        
            $polilinia=Polyline::Encode($arrayParades);
            return $polilinia;
            
        }

        //FUNCIO OBTÉ AVISOS    
         // retorna Array amb els avisos de l'expedicio
        public function obteAvisos(){
            $data=date("Y-m-d"); //Agafa data del dia d'avui, només carreguem els avisos que no han caducat (<dataFinal)            
            $query = "SELECT * FROM Avis A            
                    WHERE
                        A.idObjecte = $this->id
                    AND A.tipusObjecte = 'expedicio'
                    AND (A.dataFinal>='$data' OR A.dataFinal IS NULL)
                    ORDER BY A.dataFinal
                    ";
            $stmt = $this->conn->prepare( $query );
            $stmt->execute();
            $arrayAvisos=array();

            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                $avis = new Avis($this->conn); //Crea un objecte Avis per a cada resposta
                $avis->id = $row["id"];
                $avis->obteDadesSenseObjecte(); 
                array_push($arrayAvisos, $avis);
            }
            
            if (count($arrayAvisos)>0) {
                return $arrayAvisos; 
            } else null;           
         }

         //FUNCIO TROBA BUS
         //Funció que retorna si hi ha un bus en circulació i la seva posició
         public function busCircula(){
            //1. Busquem si el bus circula el dia i hora actual
            //Circula avui? Fem la petició a la base de dades, si retorna 1 és que si, si retorna 0 és que no
            //DATA EN FORMAT ANGLÈS
            $data = date("Y-m-d");
            $diaSetmana = diaSetmana($this->conn, $data); //Cridem funció general des de funcions.php
           
            $query = "SELECT COUNT(*) AS resposta FROM Expedicio A
            WHERE 
               A.id = :idExpedicio
               AND A.$diaSetmana=1
               AND (A.diaInici<=:data OR A.diaInici IS NULL) AND (A.diaFinal>=:data OR A.diaFinal IS NULL)"; //Pel que fa al dia Inici i dia Final tenim en compte les expedicions que són NULL
            //echo $query;
            $stmt = $this->conn->prepare($query);
           
            $stmt->bindParam(":idExpedicio", $this->id);
            $stmt->bindParam(":data", $data);

            $stmt->execute();
            $row = $stmt->fetch(PDO::FETCH_ASSOC);


            $row["resposta"]==1 ? $circula=true : $circula=false;
        

            if ($circula) {
                //1.2 Comprovem si està circulant en l'hora actual
                $hora=date("H:i"); 
                    if (date($this->trobaHoraPas($this->paradaInici))<=$hora&&date($this->trobaHoraPas($this->paradaFinal))>$hora){
                        $circula=true;
                    } else $circula=false;

                if ($circula) {
                    //2. Comprovem la posició
                    $paradesRestants = $this->paradesRestants($hora);
                    
                    if (count($paradesRestants)>0) { 
                        //3. Calculem la posició del bus, entre la parada 0 i 1
                        $horaParadaAnterior= strtotime($this->trobaHoraPas($paradesRestants[0]));
                        $horaActual=strtotime(date("H:i:s"));
                        $horaParadaSeguent= strtotime($this->trobaHoraPas($paradesRestants[1]));
                        $diferenciaTempsActual=$horaParadaSeguent-$horaActual; //segons entre les dues parades
                        $tempsEntreParades=$horaParadaSeguent-$horaParadaAnterior; //segons que queden entre l'hora actual i la parada

                        $percentatgeRestant=$diferenciaTempsActual/$tempsEntreParades; //percentatge entre les dues parades
                        $percentatgeEfectuat=1-$percentatgeRestant;

                        //3.2 Calculem la diferència i la dividim pel tros recorregut per trobar la posició (en aquesta primera versió es treballa amb rectes)
                            //LATITUD
                            $latitudInici=$paradesRestants[0]->latitud;
                            $latitudFinal=$paradesRestants[1]->latitud;

                            $latitudActual=(($latitudFinal-$latitudInici)*$percentatgeEfectuat)+$latitudInici;

                            //LONGITUD
                            $longitudInici=$paradesRestants[0]->longitud;
                            $longitudFinal=$paradesRestants[1]->longitud;
                            $longitudActual=(($longitudFinal-$longitudInici)*$percentatgeEfectuat)+$longitudInici;


                        //4.Preparem l'array de sortida
                        $resposta=array();

                        $resposta["posicioActual"] = array (
                            "percentageEfectuat" => ($percentatgeEfectuat*100),
                            "percentatgeRestant" => ($percentatgeRestant*100),
                            "latitud" => $latitudActual,
                            "longitud" => $longitudActual

                        );

                        $resposta["paradesRestants"] = array();

                        foreach ($paradesRestants as $parada) {
                            $respostaParada = array (
                                "id" => $parada->id,
                                "nom" => $parada->nom,
                                "latitud" => $parada->latitud,
                                "longitud" => $parada->longitud,
                                "horaPas" => $this->trobaHoraPas($parada)
                            );
                            array_push($resposta["paradesRestants"], $respostaParada);  

                        }

                        return $resposta;




                    } else return null;  

                } else return null; //Si no circula ara, retornem null   

            } else return null; //Si no circula avui, retornem null

         }

         //FUNCIO NUM AVISOS USUARI
         public function numAvisosUsuari(){
            $numAvisos=0;
            //Creem un objecte Avisos Usuari i obtenim els avisos d'aquesta expedició

            $avisUsuari = new AvisUsuari($this->conn);
            $avisUsuari->idExpedicio = $this->id;

            $arrayAvisos=$avisUsuari->obteAvisosExpedicio();

            $numAvisos=count($arrayAvisos);

            return $numAvisos;
         }

         //FUNCIO PARADES QUE QUEDEN
         public function paradesRestants($hora) {
            //Busquem quina és la propera parada de l'expedició
            $query = "SELECT A.idParada FROM Horari A
            WHERE 
               A.idExpedicio = :idExpedicio
               AND A.horaPas > :hora
               ORDER BY A.horaPas"; 
        
            $stmt = $this->conn->prepare($query);

            $stmt->bindParam(":idExpedicio", $this->id);
            $stmt->bindParam(":hora", $hora);

            $stmt->execute();
            $row = $stmt->fetch(PDO::FETCH_ASSOC);
            
            $properaParada=new Parada($this->conn);
            $properaParada->id = $row["idParada"];
            $properaParada->obteDades();

            $paradesRecorregut=$this->obteParades();

            //Recorrem l'array buscant les parades que sí formen part del recorregut restant
            $posicioParadaInici=null;
            $paradesRestants=array();
            
            for ($i=0;$i<count($paradesRecorregut);$i++) {

                if ($properaParada->id==$paradesRecorregut[$i]->id&&is_null($posicioParadaInici)) {
                    $posicioParadaInici=$i;
                    array_push($paradesRestants, $paradesRecorregut[$i-1]);  //Carreguem la parada anterior a la pròxima, ocuparà la posició 0.
                } 

                if (!is_null($posicioParadaInici)) {
                    array_push($paradesRestants, $paradesRecorregut[$i]);
                }


            }

            return $paradesRestants;

         }

        //FUNCIO AFEGIR
         //Funció per afegir una nova expedició. Retorna true o false, segons l'èxit de la petició
        public function afegir(){
            $linia=$this->linia->codi;
            $query = "INSERT INTO Expedicio
                            SET codiLinia=:codiLinia, dilluns=:dilluns, dimarts=:dimarts, dimecres=:dimecres, dijous=:dijous, divendres=:divendres, dissabte=:dissabte, festiu=:festiu, paradaInici=:paradaInici, paradaFinal=:paradaFinal, diaInici=:diaInici, diaFinal=:diaFinal";
        $stmt = $this->conn->prepare( $query );
    
        //Assignem els valors amagats de la petició
        $stmt->bindParam(":codiLinia", $this->linia->codi);
        $stmt->bindParam(":dilluns", $this->diesCirculacio[1]);
        $stmt->bindParam(":dimarts", $this->diesCirculacio[2]);
        $stmt->bindParam(":dimecres", $this->diesCirculacio[3]);
        $stmt->bindParam(":dijous", $this->diesCirculacio[4]);
        $stmt->bindParam(":divendres", $this->diesCirculacio[5]);
        $stmt->bindParam(":dissabte", $this->diesCirculacio[6]);
        $stmt->bindParam(":festiu", $this->diesCirculacio[0]);
        $stmt->bindParam(":paradaInici", $this->paradaInici->id);
        $stmt->bindParam(":paradaFinal", $this->paradaFinal->id);
        $stmt->bindParam(":diaInici", $this->diaInici);
        $stmt->bindParam(":diaFinal", $this->diaFinal);
        
            
        //Executem la consulta, si va bé retorna true, si no, false.
        if($stmt->execute()){
            $this->id = $this->conn->lastInsertId(); //Obtenim el codi introduit per la línia
            return true;
        }

        return false;    
    
        }


        //FUNCIO AFEGIR HORARI
        //Afegim l'horari de les parades del recorregut de la línia a partir de l'array de parades rebuda
        public function afegirHorari($parades){
            $query = "INSERT INTO Horari
                            SET idExpedicio=:idExpedicio, idParada=:idParada, horaPas=:horaPas";
        $stmt = $this->conn->prepare( $query );
        $error=false;
        //Assignem els valors amagats de la petició
        $stmt->bindParam(":idExpedicio", $this->id);
        
        //Preparem una iteració per generar les peticions a la BD
        for ($i=0; $i<count($parades)&&!$error; $i++){
            
            $stmt->bindParam(":idParada", $parades[$i]->idParada);
            $stmt->bindParam(":horaPas", $parades[$i]->horaPas);
            $error=true;

            //executem la consulta
            if($stmt->execute()){
                $error=false; //Si va bé la consulta, tornarà la variable error a ser false
            }
        }
        
        //Si tot ha anat bé, retorna true.
        if(!$error){
            return true;
        } 
     
        return false;    
    
        }

         //FUNCIO ELIMINAR
        //Funció per eliminar una expedició. Previament s'han d'haver assignat tots els valors a les variables de l'objecte
        public function eliminar(){
            $query = "DELETE FROM Expedicio WHERE id = :idExpedicio";
            $stmt = $this->conn->prepare( $query );
        
            //Assignem els valors amagats de la petició
            $stmt->bindParam(":idExpedicio", $this->id);
        
            //Executem la consulta, si va bé retorna true, si no, false.
            if($stmt->execute()){
                return true;
            }
         
            return false;    
        
        } 

        //FUNCIO CIRCULA PERIODE
        //Funció que retorna true o false segons si el bus està en el seu període de circulació

        public function circulaPeriode(){
            $dataActual=date("Y-m-d");

            if (($dataActual>=$this->diaInici||is_null($this->diaInici))&&($dataActual<=$this->diaFinal)||is_null($this->diaFinal)) return true;

            return false;
            
        }

}
?>