<?php
//CLASSE ITINERARI
// Per calcular l'itinerari:
// 1. Mirem si entre les dues parades hi ha alguna línia entre elles que les uneix.
// 2. Consultem les expedicions més pròximes a l'hora i a la data establerta.
// 3. Generem l'array de sortida.
include_once '../objects/linia.php';
include_once '../objects/parada.php';
include_once '../objects/expedicio.php';

include_once '../funcions/polyline.php';
include_once '../funcions/funcions.php';

class Itinerari {

    // connexio de la base de dades
    private $conn;
    
    //Propietats que rep
    private $origen;
    private $destinacio;
    private $data;
    private $hora;
    private $liniesOrigen;
    private $liniesDestinacio;
    private $limitResultats;
    

    //Constructor amb la connexió a la base de dades
    public function __construct($db, $origen, $destinacio, $data, $hora, $limitResultats){
        $this->conn = $db;
        $this->origen = $origen; //Rep objecte Parada
        $this->destinacio = $destinacio; //Rep objecte Parada
        $this->data = $data;
        $this->hora = $hora;
        $this->limitResultats = $limitResultats;

    }

    //TROBA LINIES ENTRE PARADES
    //Funció per trobar si hi ha linies que coincideixen entre dues parades
    private function trobaLiniesEntreParades ($parada1, $parada2) { //Rep Objecte Parada i retorna array amb linies o error
        $liniesParada1=$parada1->obteLinies(); //rep Array amb linies
        $liniesParada2=$parada2->obteLinies();
        $arrayLinies=array();

        $totalLiniesParada1=count($liniesParada1);
        $totalLiniesParada2=count($liniesParada2);

        //Recorre les arrays rebudes buscant les linies que coincideixen.
        for ($i=0; $i<$totalLiniesParada1; $i++) {
            for ($k=0; $k<$totalLiniesParada2; $k++) {
                if ($liniesParada1[$i]->codi==$liniesParada2[$k]->codi){
                    array_push($arrayLinies, $liniesParada1[$i]);
                }
            }
        }
        //echo count($arrayLinies);
        return $arrayLinies; 
      }

       //TROBA EXPEDICIONS
    //Funció per trobar les expedicions (retorn Array) en una parada segons l'hora i les línies que hem trobat
    private function trobaExpedicions($parada, $grupLinies, $data, $hora) {
        //Preparar ordre SQL
        //PER LES LÍNIES
        $ordreLinies="(";
        $primerElement=true;        
        foreach ($grupLinies as $linia) {

            if (!$primerElement) {
                $ordreLinies.=" OR ";
            }          
            
            $ordreLinies.="B.codiLinia = ";
            $ordreLinies.=$linia->codi;
            $primerElement=false;
        }
        $ordreLinies.=")";
        
        //DATA EN FORMAT ANGLÈS
        $data = date("Y-m-d", strtotime($data));
        $diaSetmana = diaSetmana($this->conn, $data); //Cridem funció general des de funcions.php
        //echo $diaSetmana;

        // //Agafa hores i minuts de la data
        // $hora = explode(":", $hora);
        // $horaNum = $hora[0];
        // $minutsNum = $hora[1];
        // echo $horaNum;
        // echo $minutsNum;
        // $hora=urlencode($hora);
        //
        $query = "SELECT A.idExpedicio FROM Horari A JOIN Expedicio B ON A.idExpedicio = B.id
             WHERE 
                A.idParada = :idParada
                AND $ordreLinies
                AND B.$diaSetmana=1
                AND A.horaPas >= :hora
                AND (B.diaInici<=:data OR B.diaInici IS NULL) AND (B.diaFinal>=:data OR B.diaFinal IS NULL)
                ORDER BY A.horaPas"; //Pel que fa al dia Inici i dia Final tenim en compte les expedicions que són NULL
        //echo $query;
        $stmt = $this->conn->prepare($query);

        $stmt->bindParam(":idParada", $parada->id);
            $stmt->bindParam(":hora", $hora);
            $stmt->bindParam(":data", $data);
        $stmt->execute();

        $arrayExpedicions=array();
        while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){ //Creem l'array amb les dades de les expedicions
            $expedicio = new Expedicio($this->conn); //Crea un objecte Expedicio per a cada resposta
            $expedicio->id = $row["idExpedicio"];
            $expedicio->obteDades(); 
            array_push($arrayExpedicions, $expedicio);
        }
        // echo " Comptador: ";
        // echo count($arrayExpedicions);
        // echo $arrayExpedicions[0]->paradaInici->nom;
        return $arrayExpedicions; //Podem accedir als atributs dels objectes Expedicio
    }

    //FUNCIÓ CALCULA ITINERARI
    //Retorna una resposta en forma d'ARRAY amb l'itinerari trobat
    public function calculaItinerari() {

        //Primer busquem quines linies hi ha entre les dues parades
        $liniesEntreParades=$this->trobaLiniesEntreParades($this->origen, $this->destinacio);
        $numLiniesEntreParades=count($liniesEntreParades);

        //Si l'array retorna algun valor, es comencen a buscar les expedicions, si no, s'han de buscar correspondències.
        if ($numLiniesEntreParades>0){
            $arrayItineraris = array();
            
            //Busca expedicions que circulin de les linies trobades en el dia i hora concret.
            $expedicions=$this->trobaExpedicions($this->origen, $liniesEntreParades, $this->data, $this->hora);
            if (count($expedicions)>0) {
                $i=0;
                foreach ($expedicions as $expedicio) {
                   
                    if ($i<$this->limitResultats) { //Limita als resultats als que es demanen              
                        $horaPasOrigen=$expedicio->trobaHoraPas($this->origen);
                        $horaPasDestinacio=$expedicio->trobaHoraPas($this->destinacio);
                        
                        if ($horaPasDestinacio&&$horaPasOrigen<$horaPasDestinacio){  //Mira el sentit de l'expedició i que efectui parada a la destinació

                            $arrayExpedicio = array (
                                "linia" => array (
                                    "codi" => $expedicio->linia->codi,
                                    "nom" => $expedicio->linia->nom,
                                    "sigles" => $expedicio->linia->sigles,
                                    "color" => $expedicio->linia->color 
                                ),
                                "expedicio" => array (
                                    "id" => $expedicio->id,
                                    "paradaInici" => $expedicio->paradaInici->nom,
                                    "paradaFinal"  => $expedicio->paradaFinal->nom,
                                ),
                                "parades" => array(
                                    "origen" =>$this->origen->nom,
                                    "origenID" =>$this->origen->id,
                                    "final" => $this->destinacio->nom,
                                    "finalID" => $this->destinacio->id
                                ),
                                "horesPas" => array (
                                    "paradaOrigen" => $horaPasOrigen,
                                    "paradaDestinacio" => $horaPasDestinacio
                                ),
                                "recorregut" => $this->obtePolilinia($expedicio->obteParadesEnRecorregut($horaPasOrigen,$horaPasDestinacio))
                            
                            );
                            $arrayExpedicio=array($arrayExpedicio); //Per mantenir la constència de les dades si hi ha transbords    
                            array_push($arrayItineraris, $arrayExpedicio);
                            $i++;    
                        }
                        
                    }
                }
                
                if (count($arrayItineraris)>0) {
                    $resposta=array(
                        "opcions"=>$arrayItineraris,
                        "status" => "OK"
                    );
                } else {
                    $resposta=array(
                        "status" => "ERROR1",
                        "missatge" => "No s'han trobat itineraris"
                    );
                }
                return $resposta;
            } else return array (
                "status" => "ERROR1",
                "missatge" => "No s'han trobat itineraris");
            
            
        } else {

                ///Si no hi ha ruta directa, s'han de buscar els transbords
                //1. Veure quines línies passen per les parades d'inici i final
                //2. Buscar estacions de correspondencia
                //3. buscar horaris seguint des d'inici a final i veure si hi ha relacions possibles.

                //PAS 1
                //Trobem linies a Parada origen ($this->origen) i parada destinació ($this->destinacio)
                $liniesParadaOrigen=$this->origen->obteLinies();
                $liniesParadaDestinacio=$this->destinacio->obteLinies();

                
                //Recorrem les dues arrays buscant les parades per linia
                $paradesLiniaOrigen=array();
                foreach ($liniesParadaOrigen as $linia) {
                    $paradesLiniaOrigen[$linia->codi]=$linia->obteParades();
                }

                $paradesLiniaDestinacio=array();
                foreach ($liniesParadaDestinacio as $linia) {
                    $paradesLiniaDestinacio[$linia->codi]=$linia->obteParades();
                }

                //PAS 2
                //Recorrem les dues arrays de linies buscant estacions coincidents
                foreach ($liniesParadaOrigen as $liniaOrigen) {
                    $arrayEstacionsCoincidents=array();   
                    for ($i=0;$i<count($paradesLiniaOrigen[$liniaOrigen->codi]);$i++) {
                        foreach ($liniesParadaDestinacio as $liniaDestinacio) {
                            for ($k=0;$k<count($paradesLiniaDestinacio[$liniaDestinacio->codi]);$k++) {
                                if ($paradesLiniaDestinacio[$liniaDestinacio->codi][$k]->id==$paradesLiniaOrigen[$liniaOrigen->codi][$i]->id){
                                    array_push($arrayEstacionsCoincidents, $paradesLiniaOrigen[$liniaOrigen->codi][$i]);  //En aquesta array emmagatzamem els objectes parada coincidents  
                                }             
                        
                            }
                        }   
                    }
                }
                
                //PAS 3
                //Generem els horaris per passos:

                //Primer de l'origen a la destinació
                $liniesEntreParades=$this->trobaLiniesEntreParades($this->origen, $arrayEstacionsCoincidents[0]);
                $numLiniesEntreParades=count($liniesEntreParades);              
                $expedicions=$this->trobaExpedicions($this->origen, $liniesEntreParades, $this->data, $this->hora);
                $arrayResposta= array();

                if (count($expedicions)>0) {
                    $i=0;
                    $arrayItineraris = array();
                    $arrayExpedicions=array();
                    foreach ($expedicions as $expedicio) {
                      
                        if ($i<$this->limitResultats) { //Limita als resultats als que es demanen              
                            $horaPasOrigen=$expedicio->trobaHoraPas($this->origen);
                            $horaPasDestinacio=$expedicio->trobaHoraPas($arrayEstacionsCoincidents[0]);
                            

                            if ($horaPasDestinacio&&$horaPasOrigen<$horaPasDestinacio){  //Mira el sentit de l'expedició i que efectui parada a la destinació
                                $arrayExpedicio[0] = array (
                                    "linia" => array (
                                        "codi" => $expedicio->linia->codi,
                                        "nom" => $expedicio->linia->nom,
                                        "sigles" => $expedicio->linia->sigles,
                                        "color" => $expedicio->linia->color 
                                    ),
                                    "expedicio" => array (
                                        "id" => $expedicio->id,
                                        "paradaInici" => $expedicio->paradaInici->nom,
                                        "paradaFinal"  => $expedicio->paradaFinal->nom,
                                    ),
                                    "parades" => array(
                                        "origen" =>$this->origen->nom,
                                        "origenID" =>$this->origen->id,
                                        "final" => $arrayEstacionsCoincidents[0]->nom,
                                        "finalID" => $arrayEstacionsCoincidents[0]->id
                                    ),
                                    "horesPas" => array (
                                        "paradaOrigen" =>$horaPasOrigen,
                                        "paradaDestinacio" => $horaPasDestinacio
                                    ),
                                    "recorregut" => $this->obtePolilinia($expedicio->obteParadesEnRecorregut($horaPasOrigen,$horaPasDestinacio))

                                ); 
                                array_push($arrayItineraris, $arrayExpedicio);
                                $i++;    
                            }
                            
                        }
                    }

                    //Mirem itinerari des de correspondència a destinació
                    
                    foreach ($arrayItineraris as $arrayExpedicio){
                        $liniesEntreParades=$this->trobaLiniesEntreParades($arrayEstacionsCoincidents[0], $this->destinacio);
                        $numLiniesEntreParades=count($liniesEntreParades);   
                        $expedicions=$this->trobaExpedicions($arrayEstacionsCoincidents[0], $liniesEntreParades, $this->data, $arrayExpedicio[0]["horesPas"]["paradaDestinacio"]);
                        
                        if (count($expedicions)>0) {    
                            $i=0;                        
                            foreach ($expedicions as $expedicio) {
                            
                                if ($i<1) { //Limita a només una expedició, la més propera a la sortida        
                                    $horaPasOrigen=$expedicio->trobaHoraPas($arrayEstacionsCoincidents[0]);
                                    $horaPasDestinacio=$expedicio->trobaHoraPas($this->destinacio);
                                    
                                    if ($horaPasDestinacio&&$horaPasOrigen<$horaPasDestinacio){  //Mira el sentit de l'expedició i que efectui parada a la destinació
                                        $arrayExpedicio2 = array (
                                            "linia" => array (
                                                "codi" => $expedicio->linia->codi,
                                                "nom" => $expedicio->linia->nom,
                                                "sigles" => $expedicio->linia->sigles,
                                                "color" => $expedicio->linia->color 
                                            ),
                                            "expedicio" => array (
                                                "id" => $expedicio->id,
                                                "paradaInici" => $expedicio->paradaInici->nom,
                                                "paradaFinal"  => $expedicio->paradaFinal->nom,
                                            ),
                                            "parades" => array(
                                                "origen" =>$arrayEstacionsCoincidents[0]->nom,
                                                "origenID" =>$arrayEstacionsCoincidents[0]->id,
                                                "final" => $this->destinacio->nom,
                                                "finalID" => $this->destinacio->id
                                            ),
                                            "horesPas" => array (
                                                "paradaOrigen" =>$horaPasOrigen,
                                                "paradaDestinacio" => $horaPasDestinacio
                                            ),
                                            "recorregut" => $this->obtePolilinia($expedicio->obteParadesEnRecorregut($horaPasOrigen,$horaPasDestinacio))
        
                                        ); 
                                        array_push($arrayExpedicio, $arrayExpedicio2);
                                        array_push($arrayResposta, $arrayExpedicio);
                                        $i++;    
                                    }
                                    
                                }
                            }

                        }
                    } 
                } 
                
                //Generem resposta segons resultats

                if (count($arrayResposta)>0) {
                    $resposta=array(
                        "opcions"=>$arrayResposta,
                        "status" => "TRANSBORD"
                    );
                } else {
                    $resposta=array(
                        "status" => "ERROR1",
                        "missatge" => "No s'han trobat itineraris"
                    );
                }

            return $resposta;
        }
        
    }


    //FUNCIÓ PER GENERAR POLILINIA
    //Ha de rebre les parades per on passa l'itinerari
    public function obtePolilinia($parades){ //Funció que retorna una polilinia codificada per Google Maps
        $arrayParades = array();
        if (count($parades)>0) {
            $i=0;
            foreach ($parades as $parada) {
                        $arrayParada = array (floatval($parada->latitud), floatval($parada->longitud));
                        array_push($arrayParades, $arrayParada);
                        $i++;                
            }
        }
    
        $polilinia=Polyline::Encode($arrayParades);
        return $polilinia;        
    }
    
}


?>