<?php
include_once '../objects/parada.php';
include_once '../objects/expedicio.php';
include_once '../objects/avis.php';

include_once '../funcions/polyline.php';

class Linia{

    //connexió a la base de dades
    private $conn;
    
    // propietats de l'objecte
    public $codi;
    public $nom;
    public $sigles;
    public $color;
    public $transport_demanda;
    public $avisos;

    //Constructor amb la connexió a la base de dades
    public function __construct($db){
        $this->conn = $db;
    }

        //FUNCIÓ OBTÉ DADES
        //Consulta a la base de dades la informació de l'objecte i l'assigna a les seves variables corresponents
        public function obteDades(){
            $query = "SELECT * FROM Linia A
                    WHERE
                        A.codi = ?
                   ";
            $stmt = $this->conn->prepare( $query );
        
            // assignarà el valor de id que rebrà a l'interrogant que hem deixat a la consulta.
            $stmt->bindParam(1, $this->codi);
            $stmt->execute();
        
            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            $row = $stmt->fetch(PDO::FETCH_ASSOC);
            
            $this->nom = $row['nom'];
            $this->sigles = $row['sigles'];
            $this->color = $row['color'];
            $this->transport_demanda = $row['transport_demanda'];
            $this->avisos = $this->obteAvisos();
            
        }


        // FUNCIÓ OBTÉ TOTES
        // carrega TOTES les línies
        public function obteTotes(){         
            $query = "SELECT A.codi FROM Linia A ORDER BY A.sigles";
            $stmt = $this->conn->prepare($query);
            $stmt->execute();
            $arrayLinies=array();

            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                $linia = new Linia($this->conn); //Crea un objecte Avis per a cada resposta
                $linia->codi = $row["codi"];
                $linia->obteDades(); 
                array_push($arrayLinies, $linia);
            }
            return $arrayLinies;         
            
        }
    
        
        //FUNCIO OBTE PARADES
        //Retorna un Array amb les parades de la línia
         public function obteParades(){
            $query = "SELECT A.idParada FROM EfectuaParada A
                        WHERE A.codiLinia = ?
                        ORDER BY A.ordre
                   ";
            $stmt = $this->conn->prepare( $query );
        
            // assignarà el valor de id que rebrà a l'interrogant que hem deixat a la consulta.
            $stmt->bindParam(1, $this->codi);
            $stmt->execute();
            $arrayParades=array();

            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                $parada = new Parada($this->conn); //Crea un objecte Parada per a cada resposta
                $parada->id = $row["idParada"];
                $parada->obteDades(); 
                array_push($arrayParades, $parada);
            }
            return $arrayParades;
            
        }

        //FUNCIÓ OBTÉ POLILINIA
        //Funció que retorna una polilinia codificada per Google Maps
        public function obtePolilinia(){ 
            $parades=$this->obteParades();
            $arrayParades = array();
            if (count($parades)>0) {
                $i=0;
                foreach ($parades as $parada) {
                            $arrayParada = array (floatval($parada->latitud), floatval($parada->longitud));
                            array_push($arrayParades, $arrayParada);
                            $i++;                
                }
            }
        
            $polilinia=Polyline::Encode($arrayParades);
            return $polilinia;
            
        }

        //FUNCIÓ OBTÉ EXPEDICIONS
        //Obté una array amb les expedicions de la linia
        public function obteExpedicions(){
            $data=date("Y-m-d"); //Agafa data del dia d'avui
            $query = "SELECT A.id FROM Expedicio A
                        JOIN Horari B ON A.id=B.idExpedicio
                        WHERE A.codiLinia = :codiLinia
                        AND A.paradaInici = B.idParada                        
                        AND (A.diaInici<=:data OR A.diaInici IS NULL) AND (A.diaFinal>=:data OR A.diaFinal IS NULL)
                        ORDER BY B.horaPas
                   ";
            $stmt = $this->conn->prepare( $query );
        
            // assignarà el valor de id que rebrà a l'interrogant que em deixat a la consulta.
            $stmt->bindParam(":codiLinia", $this->codi);
            $stmt->bindParam(":data", $data);

            $stmt->execute();
            $arrayExpedicions=array();

            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                $expedicio = new Expedicio($this->conn); //Crea un objecte Expedicio per a cada resposta
                $expedicio->id = $row["id"];
                $expedicio->obteDades(); 
                array_push($arrayExpedicions, $expedicio);
            }
            return $arrayExpedicions;
            
        }

        //FUNCIÓ OBTÉ EXPEDICIONS PER DIA
        //Obté una array amb les expedicions de la linia segons el dia de la setmana
        //El dia de la setmana ha de ser el nom rebut en lletres
        public function obteExpedicionsPerDia($diaSetmana){
            $data=date("Y-m-d"); //Agafa data del dia d'avui
            $query = "SELECT A.id FROM Expedicio A
                        JOIN Horari B ON A.id=B.idExpedicio
                        WHERE A.codiLinia = :codiLinia
                        AND A.paradaFinal = B.idParada
                        AND A.$diaSetmana = 1
                        AND (A.diaInici<=:data OR A.diaInici IS NULL) AND (A.diaFinal>=:data OR A.diaFinal IS NULL)
                        ORDER BY B.horaPas
                   ";
            $stmt = $this->conn->prepare( $query );
        
            // assignarà el valor de id que rebrà a l'interrogant que em deixat a la consulta.
            $stmt->bindParam(":codiLinia", $this->codi);
            $stmt->bindParam(":data", $data);
            $stmt->execute();
            $arrayExpedicions=array();

            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                $expedicio = new Expedicio($this->conn); //Crea un objecte Línia per a cada resposta
                $expedicio->id = $row["id"];
                $expedicio->obteDades(); 
                array_push($arrayExpedicions, $expedicio);
            }
            return $arrayExpedicions;
            
        }

         //FUNCIO OBTÉ AVISOS   
         // retorna Array amb els avisos de la linia
         public function obteAvisos(){
            $data=date("Y-m-d"); //Agafa data del dia d'avui, només carreguem els avisos que no han caducat (<dataFinal)            
            $query = "SELECT * FROM Avis A            
                    WHERE
                        A.idObjecte = :codiLinia
                    AND A.tipusObjecte = 'linia'
                    AND (A.dataFinal>=:data OR A.dataFinal IS NULL)
                    ORDER BY A.dataFinal
                   ";
            $stmt = $this->conn->prepare( $query );
            $stmt->bindParam(":codiLinia", $this->codi);
            $stmt->bindParam(":data", $data);

            $stmt->execute();
            $arrayAvisos=array();

            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                $avis = new Avis($this->conn); //Crea un objecte Avis per a cada resposta
                $avis->id = $row["id"];
                $avis->obteDadesSenseObjecte(); 
                array_push($arrayAvisos, $avis);
            }
            
            if (count($arrayAvisos)>0) {
                return $arrayAvisos; 
            } else null;   
        }

        //FUNCIO AFEGIR
        //Funció per afegir una nova línia. Previament s'han d'haver assignat tots els valors a les variables de l'objecte
        public function afegir(){
            $query = "INSERT INTO Linia
                            SET nom=:nom, sigles=:sigles, color=:color";
        $stmt = $this->conn->prepare( $query );
    
        //Assignem els valors amagats de la petició
        $stmt->bindParam(":nom", $this->nom);
        $stmt->bindParam(":sigles", $this->sigles);
        $stmt->bindParam(":color", $this->color);
        // $stmt->bindParam(":transport_demanda", "0"); //Modificar quan estigui operatiu el transport a la demanda
    
        //Executem la consulta, si va bé retorna true, si no, false.
        if($stmt->execute()){
            $this->codi = $this->conn->lastInsertId(); //Obtenim el codi introduit per la línia
            return true;
        }
     
        return false;    
    
        }

        //FUNCIO AFEGIR RECORREGUT
        //Afegim les parades del recorregut de la línia a partir de l'array de parades rebuda
        public function afegirRecorregut($idParades){
            $query = "INSERT INTO EfectuaParada
                            SET codiLinia=:codiLinia, idParada=:idParada, ordre=:ordre";
        $stmt = $this->conn->prepare( $query );
        $error=false;
        //Assignem els valors amagats de la petició
        $stmt->bindParam(":codiLinia", $this->codi);
        
        //Preparem una iteració per generar les peticions a la BD
        for ($i=1; $i<count($idParades)&&!$error; $i++){
            
            $stmt->bindParam(":idParada", $idParades[$i]);
            $stmt->bindParam(":ordre", $i);
            $error=true;

            //executem la consulta
            if($stmt->execute()){
                $error=false; //Si va bé la consulta, tornarà la variable error a ser false
            }
        }
        
        //Si tot ha anat bé, retorna true.
        if(!$error){
            return true;
        } 
     
        return false;    
    
        }

        //FUNCIO MODIFICAR
        //Funció per modificar una línia. Previament s'han d'haver assignat tots els valors a les variables de l'objecte
        public function modificar(){
            $query = "UPDATE Linia
                            SET nom=:nom, sigles=:sigles, color=:color
                            WHERE codi = :codiLinia";
        $stmt = $this->conn->prepare( $query );
    
        //Assignem els valors amagats de la petició
        $stmt->bindParam(":codiLinia", $this->codi);
        $stmt->bindParam(":nom", $this->nom);
        $stmt->bindParam(":sigles", $this->sigles);
        $stmt->bindParam(":color", $this->color);
        // $stmt->bindParam(":transport_demanda", "0"); //Modificar quan estigui operatiu el transport a la demanda
    
        //Executem la consulta, si va bé retorna true, si no, false.
        if($stmt->execute()){
            return true;
        }
     
        return false;    
    
        }

        //FUNCIO MODIFICA RECORREGUT
        //Afegim les parades del recorregut de la línia a partir de l'array de parades rebuda
        public function modificarRecorregut($parades){
        
        //Agafa parades actuals i compara
        //3 CASOS:
        ////1: Ja existeix i continua sent al recorregut --> ACTUALITZA
        ////2: Ja existeix però no és al recorregut nou --> ELIMINA
        ////3: No existeix però és al recorregut --> AFEGEIX

        $recorregutActual=$this->obteParades();
        $recorregutNou=$parades;   
        $ordre=1; 
        $error=false;
        
        //CASOS 1 i 3
        for ($i=1; $i<count($parades)&&!$error; $i++){ //Recorrem l'array nova i vella

            $cas=3;
            foreach ($recorregutActual as $paradaRecorregutActual) {
                if ($parades[$i]==$paradaRecorregutActual->id) { //Existeix al recorregut vell
                   $cas=1;     
                }
            }

            if (!$error) {
                switch ($cas){
                    case 1: //Actualitzar valor
                        $query = "UPDATE EfectuaParada
                                    SET ordre=:ordre
                                    WHERE codiLinia=:codiLinia AND idParada=:idParada";
                        $stmt = $this->conn->prepare( $query );
                        $stmt->bindParam(":codiLinia", $this->codi);
                        $stmt->bindParam(":idParada", $parades[$i]);
                        $stmt->bindParam(":ordre", $i);
                        $error=true;
    
                        //executem la consulta
                        if($stmt->execute()){
                            $error=false; //Si va bé la consulta, tornarà la variable error a ser false
                        }

                        break;
                    case 3: //Afegir valor
                        $query = "INSERT INTO EfectuaParada
                                        SET codiLinia=:codiLinia, idParada=:idParada, ordre=:ordre";
                        $stmt = $this->conn->prepare( $query );
                        $stmt->bindParam(":codiLinia", $this->codi);
                        $stmt->bindParam(":idParada", $parades[$i]);
                        $stmt->bindParam(":ordre", $i);
                        $error=true;
    
                        //executem la consulta
                        if($stmt->execute()){
                            $error=false; //Si va bé la consulta, tornarà la variable error a ser false
                        }
                        break;
                };
            } 
        }    

        // CAS 2
        foreach ($recorregutActual as $paradaRecorregutActual) {
            $cas=2;  
            for ($i=1; $i<count($parades)&&!$error; $i++){
                if ($parades[$i]==$paradaRecorregutActual->id) { //Ja existeix
                    $cas=1;     
                }
            }

            if ($cas==2&&!$error) {
                $query = "DELETE FROM EfectuaParada
                                    WHERE codiLinia=:codiLinia AND idParada=:idParada";
                $stmt = $this->conn->prepare( $query );
                $stmt->bindParam(":codiLinia", $this->codi);
                $stmt->bindParam(":idParada", $paradaRecorregutActual->id);
                $error=true;

                //executem la consulta
                if($stmt->execute()){
                    $error=false; //Si va bé la consulta, tornarà la variable error a ser false
                }

            }

        }


        //Si tot ha anat bé, retorna true.
        if(!$error){
            return true;
        } 
     
        return false;    
    
        }

        //FUNCIO ELIMINAR
        //Funció per eliminar una línia. Previament s'han d'haver assignat tots els valors a les variables de l'objecte
        public function eliminar(){
        $query = "DELETE FROM Linia WHERE codi = :codiLinia";
        $stmt = $this->conn->prepare( $query );
    
        //Assignem els valors amagats de la petició
        $stmt->bindParam(":codiLinia", $this->codi);
    
        //Executem la consulta, si va bé retorna true, si no, false.
        if($stmt->execute()){
            return true;
        }
     
        return false;    
    
        }


}
?>