<?php
include_once '../objects/linia.php';
include_once '../objects/expedicio.php';
include_once '../objects/avis.php';
include_once '../funcions/googleAPI.php'; //Inclou les funcions d'itinerari de Google Maps

include_once '../funcions/funcions.php';



class Parada {
        // connexio de la base de dades
        private $conn;

       

        // propietats de l'objecte
        public $id;
        public $nom;   
        public $longitud;
        public $latitud;    
        public $observacions;
        public $poblacio;
        public $codiPostal;
        public $esBergueda;
    




        //Constructor amb la connexió a la base de dades i les variables de latitud i longitud
        public function __construct($db){
            $this->conn = $db;
            
        }

        //FUNCIÓ OBTÉ DADES
        //Consulta a la base de dades la informació de l'objecte i l'assigna a les seves variables corresponents
        public function obteDades(){
            $query = "SELECT A.id, A.nom, A.longitud, A.latitud, A.observacions, B.nom AS poblacio, B.codiPostal, B.bergueda FROM Parada A LEFT JOIN Poblacio B ON A.codiPostal = B.codiPostal
                    WHERE
                        A.id = ?
                   ";
            $stmt = $this->conn->prepare( $query );
        
            // assignarà el valor de id que rebrà a l'interrogant que hem deixat a la consulta. (per evitar SQL Injection)
            $stmt->bindParam(1, $this->id);
            $stmt->execute();
        
            //Desmuntem les dades rebudes i les assignem a les diferents variables de l'objecte
            $row = $stmt->fetch(PDO::FETCH_ASSOC);
            
            $this->nom = $row['nom'];
            $this->longitud = number_format($row['longitud'], 5);
            $this->latitud = number_format($row['latitud'], 5);
            $this->observacions = $row['observacions'];
            $this->poblacio = $row['poblacio'];
            $this->codiPostal = $row['codiPostal'];
            $this->esBergueda = $row['bergueda']=='1' ? true : false;
            
        }

        //FUNCIÓ OBTÉ DADES
        //Consulta a la base de dades la informació de l'objecte i l'assigna a les seves variables corresponents
        public function obteParada(){
            $query = "SELECT A.id FROM Parada A
                    WHERE
                        A.nom= ?
                   ";
            $stmt = $this->conn->prepare( $query );
        
            // assignarà el valor de id que rebrà a l'interrogant que hem deixat a la consulta. (per evitar SQL Injection)
            $stmt->bindParam(1, $this->nom);
            $stmt->execute();
        
            //Desmuntem les dades rebudes i les assignem a les diferents variables de l'objecte
            $row = $stmt->fetch(PDO::FETCH_ASSOC);
            $row && $this->id = $row['id'];
                        
        }

            // funció OBTE TOTES
            // retorna una Array amb TOTES les parades de la BD
            public function obteTotes($ordre, $limit){

                // !is_set($ordre) && $ordre="ordre"; //Si no rep variable assigna'n una
                
                if($ordre=="nom") { //Segons el valor que rebem, l'array estarà ordenada per nom o per l'ordre personalitzat
                    $query = "SELECT A.id FROM Parada A JOIN Poblacio B ON A.codiPostal=B.codiPostal ORDER BY B.bergueda DESC, A.nom ASC";
                } else $query = "SELECT A.id FROM Parada A ORDER BY -A.ordre DESC, nom ASC";
                
                $limit!=null && $query.=" LIMIT 8"; //Si rebem limit limitem les respostes
                $stmt = $this->conn->prepare($query);

                // $limit!=null && $stmt->bindValue(1, $limit);

                $stmt->execute();
                $arrayParades=array();

                //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
                while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                    $parada = new Parada($this->conn); //Crea un objecte Parada per a cada resposta
                    $parada->id = $row["id"];
                    $parada->obteDades(); 
                    array_push($arrayParades, $parada);
                }
                return $arrayParades;
            }

        //FUNCIO ASSIGNA PARADA PROXIMA    
        //Funcio per trobar la parada més pròxima a partir de les coordenades rebudes
        //Assigna a l'objecte un nou ID i en carrega les seves dades.
        public function assignarParadaProxima($latitudRebuda,$longitudRebuda) {
            //Preparem la consulta
            $query = "SELECT A.id FROM Parada A WHERE A.latitud BETWEEN (:latitudRebuda-0.015) AND (:latitudRebuda+0.015) AND A.longitud BETWEEN (:longitudRebuda-0.015) AND (:longitudRebuda+0.015) LIMIT 1";
            $stmt = $this->conn->prepare( $query );
            $stmt->bindParam(":latitudRebuda", $latitudRebuda);
            $stmt->bindParam(":longitudRebuda", $longitudRebuda);

            //Executem la consulta, i si hi ha resultats, carreguem les dades de la parada
            if ($stmt->execute()) {
                $row = $stmt->fetch(PDO::FETCH_ASSOC);
                
                if ($row['id']) {
                    $this->id = $row['id'];
                    $this->obteDades();
                    
                } 
            };
        }


        //FUNCIO OBTE LÍNIES   
        //Funció que retorna un array amb les linies de la parada
            public function obteLinies(){
                
                 $query = "SELECT A.codiLinia FROM EfectuaParada A JOIN Linia B ON A.codiLinia = B.codi WHERE A.idParada=:idParada ORDER BY B.sigles";
                $stmt = $this->conn->prepare($query);
                $stmt->bindParam(":idParada", $this->id);
                                
                $stmt->execute();
                $arrayLinies=array();
                while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                    $linia = new Linia($this->conn); //Crea un objecte Línia per a cada resposta
                    $linia->codi = $row["codiLinia"];
                    $linia->obteDades(); 
                    array_push($arrayLinies, $linia);
                }
                return $arrayLinies;
                
            }

            //FUNCIÓ OBTÉ EXPEDICIONS
            //Obté una array amb les expedicions que paren a la parada
            public function obteExpedicions(){
                
                $query = "SELECT A.id
                             FROM Expedicio A
                             JOIN Horari B ON A.id = B.idExpedicio
                             WHERE B.idParada=:idParada
                             AND (A.diaInici<=:dataAvui OR A.diaInici IS NULL) AND (A.diaFinal>=:dataAvui OR A.diaFinal IS NULL)
                             ORDER BY B.horaPas";
                $stmt = $this->conn->prepare($query);
              
               //Assignem els valors amagats de la petició
                $stmt->bindParam(":idParada", $this->id);

                $data = date("Y-m-d");
                $stmt->bindParam(":dataAvui", $data);

                if ($stmt->execute()) {
                    $arrayExpedicions=array();

                    while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                        $expedicio = new Expedicio($this->conn); //Crea un objecte Expedicio per a cada resposta
                        $expedicio->id = $row["id"];
                        $expedicio->obteDades(); 
                        array_push($arrayExpedicions, $expedicio);
                    }
                    
                    return $arrayExpedicions;

                } else return null;
               
               
               
           }

           //FUNCIÓ OBTÉ PROPERES SORTIDES
           //Retorna una Array amb les expedicions més pròximes a la data actual de la parada
            public function obteProperesSortides() {
                
                    //PER LES LÍNIES
                    //Preparem l'ordre SQL que inclogui els codis de les linies rebudes
                $grupLinies=$this->obteLinies();
                $ordreLinies="(";
                $primerElement=true;        
                foreach ($grupLinies as $linia) {

                    if (!$primerElement) {
                        $ordreLinies.=" OR ";
                    }          
                    
                    $ordreLinies.="B.codiLinia = ";
                    $ordreLinies.=$linia->codi;
                    $primerElement=false;
                }
                $ordreLinies.=")";

                //DATA EN FORMAT ANGLÈS
                $data = date("Y-m-d");
                $diaSetmana = diaSetmana($this->conn, $data); //Cridem funció general des de funcions.php
                
                $hora=date("H:i"); //CANVIAR PER HORA REAL
                $hora = date("H:i", strtotime("now - 15 minutes")); //Manté les expedicions de 15 minuts passades
               
                $query = "SELECT A.idExpedicio FROM Horari A JOIN Expedicio B ON A.idExpedicio = B.id
                WHERE 
                   A.idParada = :idParada
                   AND NOT A.idParada=B.paradaFinal
                   AND $ordreLinies
                   AND B.$diaSetmana=1
                   AND A.horaPas >= :hora
                   AND (B.diaInici<=:data OR B.diaInici IS NULL) AND (B.diaFinal>=:data OR B.diaFinal IS NULL)
                   ORDER BY A.horaPas"; //Pel que fa al dia Inici i dia Final tenim en compte les expedicions que són NULL
                //echo $query;
                $stmt = $this->conn->prepare($query);

                //Assignem els valors amagats
                $stmt->bindParam(":idParada", $this->id);
                $stmt->bindParam(":hora", $hora);
                $stmt->bindParam(":data", $data);

                $stmt->execute();
        
                $arrayExpedicions=array();
                while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){ //Creem l'array amb les dades de les expedicions
                    $expedicio = new Expedicio($this->conn); //Crea un objecte Expedicio per a cada resposta
                    $expedicio->id = $row["idExpedicio"];
                    $expedicio->obteDades(); 
                    array_push($arrayExpedicions, $expedicio);
                }
                // echo " Comptador: ";
                // echo count($arrayExpedicions);
                // echo $arrayExpedicions[0]->paradaInici->nom;
                return $arrayExpedicions; //Podem accedir als atributs dels objectes Expedicio
            }

            //FUNCIO OBTE ORDRE
            // Retorna el valor de l'ordre de la parada en una línia concreta
            public function obteOrdre($linia) {
                $query = "SELECT B.ordre FROM Parada A JOIN EfectuaParada B ON A.id=B.idParada WHERE B.codiLinia=:codiLinia AND A.id=:idParada";   
                $stmt = $this->conn->prepare($query);

                //Assignem els valors amagats
                $stmt->bindParam(":idParada", $this->id);
                $stmt->bindParam(":codiLinia", $linia->codi);

                $stmt->execute();
                $resposta = $stmt->fetch(PDO::FETCH_ASSOC);   
                return $resposta["ordre"];
            }

            //FUNCIO OBTÉ AVISOS
            // retorna Array amb els avisos de la parada
            public function obteAvisos(){
                $data=date("Y-m-d"); //Agafa data del dia d'avui, només carreguem els avisos que no han caducat (<dataFinal)            
                $query = "SELECT * FROM Avis A            
                        WHERE
                            A.idObjecte = :idParada
                        AND A.tipusObjecte = 'parada'
                        AND (A.dataFinal>=:data OR A.dataFinal IS NULL)
                        ORDER BY A.dataFinal
                        ";
                $stmt = $this->conn->prepare( $query );

                //Assignem els valors amagats
                $stmt->bindParam(":idParada", $this->id);
                $stmt->bindParam(":data", $data);

                $stmt->execute();
                $arrayAvisos=array();

                //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
                while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                    $avis = new Avis($this->conn); //Crea un objecte Avis per a cada resposta
                    $avis->id = $row["id"];
                    $avis->obteDadesSenseObjecte(); 
                    array_push($arrayAvisos, $avis);
                }
                
                if (count($arrayAvisos)>0) {
                    return $arrayAvisos; 
                } else null;
                           
            }

            //FUNCIO TROBA CODI POSTAL
            //Funció que troba el codi postal a partir de les coordenades rebudes
            public function trobaCodiPostal(){
                $respostaGoogleMaps = consultaCoordenades($this->latitud, $this->longitud);
                
                foreach (($respostaGoogleMaps["results"][0]["address_components"]) as $resultat) {
                    $resultat["types"][0]=="postal_code" && $this->codiPostal = $resultat["long_name"];
                }
            }

            //FUNCIO MIRA SI POBLACIO EXISTEIX A BD
            //Funció que busca si el codiPostal està a la BD
            public function existeixPoblacio(){
               //1. Mira si la població existeix a la BD.
               $query = "SELECT COUNT(*) as 'trobat' FROM Poblacio WHERE codiPostal=:codiPostal";
                $stmt = $this->conn->prepare( $query );

                $stmt->bindParam(":codiPostal", $this->codiPostal);

                $stmt->execute();

                $row = $stmt->fetch(PDO::FETCH_ASSOC);
                
                if ($row["trobat"]>0) {
                    return true;
                } else return false;

            }

            //FUNCIO QUE AFEGEIX POBLACIÓ A LA BD
            //Funció que busca si el codiPostal està a la BD
            public function afegeixPoblacio(){
                //1. Obtenim les dades necessaries 
                $respostaGoogleMaps = consultaLloc($this->codiPostal);
                $nomPoblacio=$respostaGoogleMaps["results"][0]["address_components"][1]["long_name"];

                $query = "INSERT INTO Poblacio
                                    SET codiPostal=:codiPostal, nom=:nom";
                 $stmt = $this->conn->prepare( $query );
 
                 $stmt->bindParam(":codiPostal", $this->codiPostal);
                 $stmt->bindParam(":nom", $nomPoblacio);
 
                 if($stmt->execute()){
                    return true;
                }             
                return false;   
 
             }




            //FUNCIO AFEGIR
            //Funció per afegir una nova parada. Previament s'han d'haver assignat tots els valors a les variables de l'objecte
            public function afegir(){
                $query = "INSERT INTO Parada
                                SET nom=:nom, latitud=:latitud, longitud=:longitud, codiPostal=:codiPostal, observacions=:observacions";
            $stmt = $this->conn->prepare( $query );
        
            //Afegim la població a la taula corresponent, en cas que no existeixi
            !$this->existeixPoblacio() && $this->afegeixPoblacio();
            

            //Assignem els valors amagats de la petició
            $stmt->bindParam(":nom", $this->nom);
            $stmt->bindParam(":latitud", $this->latitud);
            $stmt->bindParam(":longitud", $this->longitud);
            $stmt->bindParam(":codiPostal", $this->codiPostal);
            $stmt->bindParam(":observacions", $this->observacions);
        
            //Executem la consulta, si va bé retorna true, si no, false.
            if($stmt->execute()){
                $this->id = $this->conn->lastInsertId(); //Obtenim l'id de la parada introduida
                return true;
            }
         
            return false;    
        
            }


            //FUNCIÓ BUSCA EXPEDICIO ENTRE DUES PARADES
            //Retorna una array amb les expedicions compartides entre dues parades
            public function expedicionsEntreParades($parada1, $parada2){
                
                //Obtenim les expedicions que passen per cada Parada
                $expedicionsParada1=$parada1->obteExpedicions();
                $expedicionsParada2=$parada2->obteExpedicions();
            
                $expedicionsCompartides = array();
                //Busquem les expedicions que s'aturen a les dues parades, recorrent les dues arrays d'expedicions
                foreach ($expedicionsParada1 as $expedicioParada1) {
                    foreach ($expedicionsParada2 as $expedicioParada2) {
                        if ($expedicioParada1->id==$expedicioParada2->id) {
                            array_push($expedicionsCompartides, $expedicioParada1);
                        }    
                    }
                }

                return $expedicionsCompartides;           
            
            
            }

                    //FUNCIO MODIFICAR
        //Funció per modificar una parada. Previament s'han d'haver assignat tots els valors a les variables de l'objecte
        public function modificar(){
            $query = "UPDATE Parada
                            SET nom=:nom, latitud=:latitud, longitud=:longitud, codiPostal=:codiPostal, observacions=:observacions
                            WHERE id = :idParada";
        $stmt = $this->conn->prepare( $query );
    
        //Assignem els valors amagats de la petició
        $stmt->bindParam(":idParada", $this->id);
        $stmt->bindParam(":nom", $this->nom);
        $stmt->bindParam(":latitud", $this->latitud);
        $stmt->bindParam(":longitud", $this->longitud);
        $stmt->bindParam(":codiPostal", $this->codiPostal);
        $stmt->bindParam(":observacions", $this->observacions);
        // $stmt->bindParam(":transport_demanda", "0"); //Modificar quan estigui operatiu el transport a la demanda
    
        //Executem la consulta, si va bé retorna true, si no, false.
        if($stmt->execute()){
            return true;
        }
     
        return false;    
    
        }

            //FUNCIO ELIMINAR
        //Funció per eliminar una parada. Previament s'han d'haver assignat tots els valors a les variables de l'objecte
        public function eliminar(){
            $query = "DELETE FROM Parada WHERE id = :idParada";
            $stmt = $this->conn->prepare( $query );
        
            //Assignem els valors amagats de la petició
            $stmt->bindParam(":idParada", $this->id);
        
            //Executem la consulta, si va bé retorna true, si no, false.
            if($stmt->execute()){
                return true;
            }
         
            return false;    
        
            }

}
?>