<?php
// Classe Usuari
class Usuari {
 
    // connexió a la base de dades
    private $conn;
 
    // propietats de l'objecte, corresponen a les mateixes que a la BD
    public $id;
    public $nom;
    public $email;
    public $contrasenya;

 
    // Constructor amb la crida a la BD
    public function __construct($db){
        $this->conn = $db;
    }
 

        //FUNCIÓ PER CREAR USUARI
    function crear(){ 

        // Preparem la consulta SQL
        $query = "INSERT INTO Usuaris
                SET
                    nom = :nom,
                    email = :email,
                    contrasenya = :contrasenya";
        $stmt = $this->conn->prepare($query);
    
        // netejem el codi rebut
        $this->nom=htmlspecialchars(strip_tags($this->nom));
        $this->email=htmlspecialchars(strip_tags($this->email));
        $this->contrasenya=htmlspecialchars(strip_tags($this->contrasenya));
    
        // Assignem els valors SQL
        $stmt->bindParam(':nom', $this->nom);
        $stmt->bindParam(':email', $this->email);
    
        // codifiquem la contrasenya
        $password_hash = password_hash($this->contrasenya, PASSWORD_BCRYPT);
        $stmt->bindParam(':contrasenya', $password_hash);
    
        // execute the query, also check if query was successful
        if($stmt->execute()){
            return true;
        } 
        return false;
    }

    //FUNCIÓ EXISTEIX USUARI
    // Mira si l'usuari passat existeix i agafa les dades de la BD
    function existeixUsuari(){
 
    // Preparem la consulta que es fa a la BD
    $query = "SELECT id, nom, contrasenya
            FROM Usuaris
            WHERE email = ?
            LIMIT 0,1";

    $stmt = $this->conn->prepare( $query );
 
    // netejem la direcció rebuda
    $this->email=htmlspecialchars(strip_tags($this->email));
 
    // assignem els valors SQL
    $stmt->bindParam(1, $this->email);
 
    // Executem la consulta
    $stmt->execute();
 
    // Obtenim el número de files. Si és 1 s'ha trobat l'usuari, si és 0, no.
    $num = $stmt->rowCount();
 
    // Si hi ha l'usuari, assignem les dades rebudes a les variables de la classe corresponents.
    if($num>0){
 
        // Com que només rebem una fila, no cal fer iteracions, l'assignem directament.
        $row = $stmt->fetch(PDO::FETCH_ASSOC);
 
        // assignem els valors rebuts a les variables de l'objecte
        $this->id = $row['id'];
        $this->nom = $row['nom'];
        $this->contrasenya = $row['contrasenya'];
 
        // tornem true perquè l'usuari existeix i el procediment ha anat bé
        return true;
    }
 
    // tornem false si no s'ha trobat cap correu electrònic.
    return false;
}

}
?>