<?php
// Capçaleres per tal que funcioni l'API
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Headers: access");
header("Access-Control-Allow-Methods: GET");
header("Access-Control-Allow-Credentials: true");
header('Content-Type: application/json');
 
// Agafa l'objecte que genera l'API i les dades de la BD
include_once '../config/database.php';
include_once '../objects/parada.php';
include_once '../objects/expedicio.php';


// Prepara la connexió a la base de dades
$database = new Database();
$db = $database->getConnection();

// crea un nou objecte
$paradaOrigen = new Parada($db);
$paradaDestinacio = new Parada($db);


// Agafa l'ID de la linia i els resultats sol·licitats a partir del rebut a través del GET (.php?id=...)
$paradaOrigen->id = isset($_GET['paradaOrigen']) ? $_GET['paradaOrigen'] : null;
$paradaDestinacio->id=isset($_GET['paradaDestinacio']) ? $_GET['paradaDestinacio'] : null; //Mirem quin ordre volem generar
$diaSetmana=isset($_GET['diaSetmana']) ? $_GET['diaSetmana'] : null; //Mirem quin dia volem generar

if($paradaOrigen->id!=null&&$paradaDestinacio->id!=null){ //Mirem que hem rebut les dades obligatories

    //Obtenim les dades de les parades
    $paradaOrigen->obteDades();
    $paradaDestinacio->obteDades();

    if ($paradaOrigen->nom!=null&&$paradaDestinacio->nom!=null) { //Comprovem que les dues parades rebudes, existeixen

        //Busquem les expedicions que uneixen les dues poblacions
        $expedicionsCompartides=$paradaOrigen->expedicionsEntreParades($paradaOrigen, $paradaDestinacio);

        if ($expedicionsCompartides!=null) { //Comprovem que s'han trobat expedicions compartides
            //ENDREÇA DE EXPEDICIONS
            //1. PRIMER ELS CLASSIFIQUEM PER SENTITS
            $expedicionsASC=array();
            $expedicionsDESC=array();

            for ($i=0; $i<count($expedicionsCompartides); $i++) {
                $horaPasOrigen=$expedicionsCompartides[$i]->trobaHoraPas($paradaOrigen);
                $horaPasDestinacio=$expedicionsCompartides[$i]->trobaHoraPas($paradaDestinacio);

                if ($horaPasOrigen<$horaPasDestinacio) { //Si l'hora d'origen és més baixa que la de destinació, el sentit és DESCENDENT (origen->destinació)
                    array_push($expedicionsDESC, $expedicionsCompartides[$i]);
                } else array_push($expedicionsASC, $expedicionsCompartides[$i]); //Si l'hora d'origen és més alta que la de destinació, el sentit és ASCENDENT (destinació->origen)

            }  
            
            //2. CLASSIFIQUEM PER DIES DE LA SETMANA
            $expedicionsDESCSetmana=classificaPerDia($expedicionsDESC, 2);
            $expedicionsDESCDissabte=classificaPerDia($expedicionsDESC, 6);
            $expedicionsDESCDiumenge=classificaPerDia($expedicionsDESC, 0);

            $expedicionsASCSetmana=classificaPerDia($expedicionsASC, 2);
            $expedicionsASCDissabte=classificaPerDia($expedicionsASC, 6);
            $expedicionsASCDiumenge=classificaPerDia($expedicionsASC, 0);

            //PREPAREM ARRAY DE SORTIDA
            $resposta=array(
                        "dades" => array(
                            "estacioOrigen" => array (
                                "id" => $paradaOrigen->id,
                                "nom" => $paradaOrigen->nom
                            ),
                            "estacioDestinacio" => array (
                                "id" => $paradaDestinacio->id,
                                "nom" => $paradaDestinacio->nom
                            )
                        ),
                        "expedicions" => array(
                            "anada" => array(
                                "setmana" => generaRespostaExpedicions($expedicionsDESCSetmana, $paradaOrigen, $paradaDestinacio),
                                "dissabte" => generaRespostaExpedicions($expedicionsDESCDissabte, $paradaOrigen, $paradaDestinacio),
                                "festius" => generaRespostaExpedicions($expedicionsDESCDiumenge, $paradaOrigen, $paradaDestinacio)
                            ),
                            "tornada" => array(
                                "setmana" => generaRespostaExpedicions($expedicionsASCSetmana, $paradaDestinacio, $paradaOrigen),
                                "dissabte" => generaRespostaExpedicions($expedicionsASCDissabte, $paradaDestinacio, $paradaOrigen),
                                "festius" => generaRespostaExpedicions($expedicionsASCDiumenge, $paradaDestinacio, $paradaOrigen)
                            ),
                            
                        )                        
                    );
            
            // Si alguna parada no existeix, retorna un missatge d'error
            http_response_code(200);
            echo json_encode($resposta);    
        } else {
            // Si no hi ha serveis entre les dues parades, retorna un missatge d'error
            http_response_code(404);
            echo json_encode(array("missatge" => "No s'han trobat expedicions entre les dues parades"));

        }




        
    
    
    
    } else {
        // Si alguna parada no existeix, retorna un missatge d'error
        http_response_code(404);
        echo json_encode(array("missatge" => "Alguna de les parades introduides, no existeix"));
    }



} else  { // Si falten dades, retorna un missatge d'error
    http_response_code(404);
    echo json_encode(array("missatge" => "Cal omplir totes les dades"));
    }


//FUNCIO QUE CLASSIFICA LES EXPEDICIONS REBUDES SEGONS EL DIA
function classificaPerDia($arrayExpedicions, $diaSetmana) { //0 per diumenge
    $arrayResposta=array();
    foreach ($arrayExpedicions as $expedicio) {
        if ($expedicio->diesCirculacio[$diaSetmana]=="1") {
            array_push($arrayResposta, $expedicio);
        }

    }
    if (count($arrayResposta)==0) return null;
    return $arrayResposta;
}

function generaRespostaExpedicions($arrayExpedicions, $parada1, $parada2) { //0 per diumenge
    $arrayResposta=array();
   foreach ($arrayExpedicions as $expedicio) {
        $expedicioResposta=array(
                "id" => $expedicio->id,
                "origenExpedicio" =>$expedicio->paradaInici->nom,
                "destinacioExpedicio" =>$expedicio->paradaFinal->nom,
                "linia" => array (
                    "codi" => $expedicio->linia->codi,
                    "nom" =>$expedicio->linia->nom,
                    "sigles" => $expedicio->linia->sigles,
                    "color" => $expedicio->linia->color 
                ),
                "horaSortida" => $expedicio->trobaHoraPas($parada1),
                "horaArribada" => $expedicio->trobaHoraPas($parada2)
            );
        array_push($arrayResposta, $expedicioResposta);
        }

    if (count($arrayResposta)==0) return null;
    return $arrayResposta;
}
?>