<?php

include_once '../objects/linia.php';
include_once '../objects/parada.php';
include_once '../objects/avis.php';

include_once '../funcions/polyline.php';

class Expedicio {

    //connexió a la base de dades
    private $conn;

    // propietats de l'objecte
    public $id;
    public $linia;
    public $invers;
    public $paradaInici;
    public $paradaFinal;
    public $diesCirculacio; //0 és diumenge
    public $horari;
    public $diaInici;
    public $diaFinal;

     //Constructor amb la connexió a la base de dades
     public function __construct($db){
        $this->conn = $db;
    }

    // funcio readOne
        // carrega informació d'una línia concreta
        public function obteDades(){
            $query = "SELECT * FROM Expedicio A
                    WHERE
                        A.id = ?
                   ";
            $stmt = $this->conn->prepare( $query );
        
            // assignarà el valor de id que rebrà a l'interrogant que em deixat a la consulta.
            $stmt->bindParam(1, $this->id);
            $stmt->execute();
        
            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            $row = $stmt->fetch(PDO::FETCH_ASSOC);
            
            $this->linia = new Linia($this->conn);
            $this->linia->codi = $row['codiLinia'];
            $this->linia->obteDades(); 
            
            $this->paradaInici = new Parada($this->conn);
            $this->paradaInici->id = $row['paradaInici'];
            $this->paradaInici->obteDades();
            $this->paradaFinal = new Parada($this->conn);
            $this->paradaFinal->id = $row['paradaFinal'];
            $this->paradaFinal->obteDades();

            $this->diesCirculacio=array($row['festiu'],$row['dilluns'],$row['dimarts'],$row['dimecres'],$row['dijous'],$row['divendres'],$row['dissabte']);

            $this->diaInici = ($row['diaInici']!=null ? date("Y-m-d", strtotime($row['diaInici'])) : null);
            $this->diaFinal = ($row['diaFinal']!=null ? date("Y-m-d", strtotime($row['diaFinal'])) : null);

            //Calculem invers segons l'ordre de la línia
            if ($this->paradaInici->obteOrdre($this->linia)<$this->paradaFinal->obteOrdre($this->linia)){
                $this->invers="DESC";
            } else  $this->invers="ASC";
            
        }

        public function arrayHoraPas() {
            $query = "SELECT * FROM Horari A
                    WHERE
                        A.idExpedicio = ?
                        ORDER BY A.horaPas
                   ";
            $stmt = $this->conn->prepare( $query );
        
            // assignarà el valor de id que rebrà a l'interrogant que em deixat a la consulta.
            $stmt->bindParam(1, $this->id);
            $stmt->execute();
            $arrayParades=array();
            while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                $parada= new Parada($this->conn); //Crea un objecte Línia per a cada resposta
                $parada->id = $row["idParada"];
                $parada->obteDades();
                $horaPas = date("H:i", strtotime($row["horaPas"]));

                $arrayParada=array($parada, $horaPas);
                array_push($arrayParades, $arrayParada);
            }
            return $arrayParades;

        }

        public function obteParades(){
            $query = "SELECT A.idParada FROM Horari A
                        WHERE A.idExpedicio = ?
                        ORDER BY A.horaPas
                   ";
            $stmt = $this->conn->prepare( $query );
        
            // assignarà el valor de id que rebrà a l'interrogant que em deixat a la consulta.
            $stmt->bindParam(1, $this->id);
            $stmt->execute();
            $arrayParades=array();

            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                $parada = new Parada($this->conn); //Crea un objecte Línia per a cada resposta
                $parada->id = $row["idParada"];
                $parada->obteDades(); 
                array_push($arrayParades, $parada);
            }
            return $arrayParades;            
        }

        public function trobaHoraPas($parada) {
            $query = "SELECT * FROM Horari A
                    WHERE
                        A.idExpedicio = ?
                    AND
                        A.idParada = ?
                   ";
            $stmt = $this->conn->prepare( $query );
        
            // assignarà el valor de id que rebrà a l'interrogant que em deixat a la consulta.
            $stmt->bindParam(1, $this->id);
            $stmt->bindParam(2, $parada->id);
            $stmt->execute();
        
            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            $resposta = $stmt->fetch(PDO::FETCH_ASSOC);
            if (!is_null($resposta["horaPas"])){ 
                $horaPas = date("H:i", strtotime($resposta["horaPas"]));
            } else $horaPas=null;
            
            return $horaPas;

        }

        public function obtePolilinia(){ //Funció que retorna una polilinia codificada per Google Maps
            $parades=$this->obteParades();
            $arrayParades = array();
            if (count($parades)>0) {
                $i=0;
                foreach ($parades as $parada) {
                            $arrayParada = array (floatval($parada->latitud), floatval($parada->longitud));
                            array_push($arrayParades, $arrayParada);
                            $i++;                
                }
            }
        
            $polilinia=Polyline::Encode($arrayParades);
            return $polilinia;
            
        }


        // retorna Array amb els avisos de la linia
        public function obteAvisos(){
            $data=date("Y-m-d"); //Agafa data del dia d'avui, només carreguem els avisos que no han caducat (<dataFinal)            
            $query = "SELECT * FROM Avis A            
                    WHERE
                        A.idObjecte = $this->id
                    AND A.tipusObjecte = 'expedicio'
                    AND (A.dataFinal>='$data' OR A.dataFinal IS NULL)
                    ORDER BY A.dataFinal
                    ";
            $stmt = $this->conn->prepare( $query );
            $stmt->execute();
            $arrayAvisos=array();

            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                $avis = new Avis($this->conn); //Crea un objecte Avis per a cada resposta
                $avis->id = $row["id"];
                $avis->obteDadesSenseObjecte(); 
                array_push($arrayAvisos, $avis);
            }
            
            if (count($arrayAvisos)>0) {
                return $arrayAvisos; 
            } else null;           
         }
}
?>