<?php
include_once '../objects/parada.php';
include_once '../objects/expedicio.php';
include_once '../objects/avis.php';

include_once '../funcions/polyline.php';

class Linia{

    //connexió a la base de dades
    private $conn;
    
    // propietats de l'objecte
    public $codi;
    public $nom;
    public $sigles;
    public $color;
    public $transport_demanda;
    public $avisos;

    //Constructor amb la connexió a la base de dades
    public function __construct($db){
        $this->conn = $db;
    }


    // funció llegir
    // carrega TOTES les línies
    public function obteTotes(){
         
        $query = "SELECT * FROM Linia A";
        $stmt = $this->conn->prepare($query);
        $stmt->execute();
        $arrayLinies=array();

        //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
        while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
            $linia = new Linia($this->conn); //Crea un objecte Avis per a cada resposta
            $linia->codi = $row["codi"];
            $linia->obteDades(); 
            array_push($arrayLinies, $linia);
        }
        return $arrayLinies;         
        
    }
    
        // funcio readOne
        // carrega informació d'una línia concreta
        public function obteDades(){
            $query = "SELECT * FROM Linia A
                    WHERE
                        A.codi = ?
                   ";
            $stmt = $this->conn->prepare( $query );
        
            // assignarà el valor de id que rebrà a l'interrogant que em deixat a la consulta.
            $stmt->bindParam(1, $this->codi);
            $stmt->execute();
        
            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            $row = $stmt->fetch(PDO::FETCH_ASSOC);
            
            $this->nom = $row['nom'];
            $this->sigles = $row['sigles'];
            $this->color = $row['color'];
            $this->transport_demanda = $row['transport_demanda'];
            $this->avisos = $this->obteAvisos();
            
        }
        
    public function obteParades(){
            $query = "SELECT A.idParada FROM EfectuaParada A
                        WHERE A.codiLinia = ?
                        ORDER BY A.ordre
                   ";
            $stmt = $this->conn->prepare( $query );
        
            // assignarà el valor de id que rebrà a l'interrogant que em deixat a la consulta.
            $stmt->bindParam(1, $this->codi);
            $stmt->execute();
            $arrayParades=array();

            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                $parada = new Parada($this->conn); //Crea un objecte Línia per a cada resposta
                $parada->id = $row["idParada"];
                $parada->obteDades(); 
                array_push($arrayParades, $parada);
            }
            return $arrayParades;
            
        }

        public function obtePolilinia(){ //Funció que retorna una polilinia codificada per Google Maps
            $parades=$this->obteParades();
            $arrayParades = array();
            if (count($parades)>0) {
                $i=0;
                foreach ($parades as $parada) {
                            $arrayParada = array (floatval($parada->latitud), floatval($parada->longitud));
                            array_push($arrayParades, $arrayParada);
                            $i++;                
                }
            }
        
            $polilinia=Polyline::Encode($arrayParades);
            return $polilinia;
            
        }

        public function obteExpedicions(){
            $data=date("Y-m-d"); //Agafa data del dia d'avui
            $query = "SELECT A.id FROM Expedicio A
                        JOIN Horari B ON A.id=B.idExpedicio
                        WHERE A.codiLinia = ?
                        AND A.paradaInici = B.idParada                        
                        AND (A.diaInici<='$data' OR A.diaInici IS NULL) AND (A.diaFinal>='$data' OR A.diaFinal IS NULL)
                        ORDER BY B.horaPas
                   ";
            $stmt = $this->conn->prepare( $query );
        
            // assignarà el valor de id que rebrà a l'interrogant que em deixat a la consulta.
            $stmt->bindParam(1, $this->codi);
            $stmt->execute();
            $arrayExpedicions=array();

            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                $expedicio = new Expedicio($this->conn); //Crea un objecte Línia per a cada resposta
                $expedicio->id = $row["id"];
                $expedicio->obteDades(); 
                array_push($arrayExpedicions, $expedicio);
            }
            return $arrayExpedicions;
            
        }

        public function obteExpedicionsPerDia($diaSetmana){
            $data=date("Y-m-d"); //Agafa data del dia d'avui
            $query = "SELECT A.id FROM Expedicio A
                        JOIN Horari B ON A.id=B.idExpedicio
                        WHERE A.codiLinia = ?
                        AND A.paradaInici = B.idParada
                        AND A.$diaSetmana = 1
                        AND (A.diaInici<='$data' OR A.diaInici IS NULL) AND (A.diaFinal>='$data' OR A.diaFinal IS NULL)
                        ORDER BY B.horaPas
                   ";
            $stmt = $this->conn->prepare( $query );
        
            // assignarà el valor de id que rebrà a l'interrogant que em deixat a la consulta.
            $stmt->bindParam(1, $this->codi);
            $stmt->execute();
            $arrayExpedicions=array();

            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                $expedicio = new Expedicio($this->conn); //Crea un objecte Línia per a cada resposta
                $expedicio->id = $row["id"];
                $expedicio->obteDades(); 
                array_push($arrayExpedicions, $expedicio);
            }
            return $arrayExpedicions;
            
        }


         // retorna Array amb els avisos de la linia
         public function obteAvisos(){
            $data=date("Y-m-d"); //Agafa data del dia d'avui, només carreguem els avisos que no han caducat (<dataFinal)            
            $query = "SELECT * FROM Avis A            
                    WHERE
                        A.idObjecte = $this->codi
                    AND A.tipusObjecte = 'linia'
                    AND (A.dataFinal>='$data' OR A.dataFinal IS NULL)
                    ORDER BY A.dataFinal
                   ";
            $stmt = $this->conn->prepare( $query );
            $stmt->execute();
            $arrayAvisos=array();

            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                $avis = new Avis($this->conn); //Crea un objecte Avis per a cada resposta
                $avis->id = $row["id"];
                $avis->obteDadesSenseObjecte(); 
                array_push($arrayAvisos, $avis);
            }
            
            if (count($arrayAvisos)>0) {
                return $arrayAvisos; 
            } else null;   
        }
}
?>