<?php
include_once '../objects/linia.php';
include_once '../objects/expedicio.php';
include_once '../objects/avis.php';

include_once '../funcions/funcions.php';



class Parada {
        // connexio de la base de dades
        private $conn;

       

        // propietats de l'objecte
        public $id;
        public $nom;   
        public $longitud;
        public $latitud;    
        public $observacions;
        public $poblacio;
        public $codiPostal;
        public $esBergueda;
    




        //Constructor amb la connexió a la base de dades i les variables de latitud i longitud
        public function __construct($db){
            $this->conn = $db;
            
        }


        //Getters i setters
        public function getLatitud(){
            return $this->latitud;
        }

        public function setLatitud($latitud){
            $this->latitud=$latitud;
        }

        public function getLongitud(){
            return $this->longitud;
        }

        public function setLongitud($longitud){
            $this->longitud=$longitud;
        }


        //Funcio per trobar la parada més pròxima a partir de les coordenades rebudes

        public function assignarParadaProxima($latitudRebuda,$longitudRebuda) {
            $query = "SELECT A.id FROM Parada A WHERE A.latitud BETWEEN ($latitudRebuda-0.015) AND ($latitudRebuda+0.015) AND A.longitud BETWEEN ($longitudRebuda-0.015) AND ($longitudRebuda+0.015) LIMIT 1";
            //echo $query . "</br>";
            $stmt = $this->conn->prepare( $query );
            $stmt->execute();
            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            $row = $stmt->fetch(PDO::FETCH_ASSOC);
            $this->id = $row['id'];
            //$this->id=1;
            $this->obteDades();
        }


                // funció llegir
            // carrega TOTES les parades
            public function obteTotes($ordre){

                // !is_set($ordre) && $ordre="ordre"; //Si no rep variable assigna'n una
                
                if($ordre=="nom") {
                    $query = "SELECT * FROM Parada A JOIN Poblacio B ON A.codiPostal=B.codiPostal ORDER BY B.bergueda DESC, A.nom ASC";
                } else $query = "SELECT * FROM Parada A ORDER BY -A.ordre DESC, id DESC";
                $stmt = $this->conn->prepare($query);
                $stmt->execute();
                $arrayParades=array();

                //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
                while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                    $parada = new Parada($this->conn); //Crea un objecte Avis per a cada resposta
                    $parada->id = $row["id"];
                    $parada->obteDades(); 
                    array_push($arrayParades, $parada);
                }
                return $arrayParades;         
                
            }

        public function obteDades(){

            $query = "SELECT A.id, A.nom, A.longitud, A.latitud, A.observacions, B.nom AS poblacio, B.codiPostal, B.bergueda FROM Parada A LEFT JOIN Poblacio B ON A.codiPostal = B.codiPostal
                    WHERE
                        A.id = ?
                   ";
            $stmt = $this->conn->prepare( $query );
        
            // assignarà el valor de id que rebrà a l'interrogant que em deixat a la consulta.
            $stmt->bindParam(1, $this->id);
            $stmt->execute();
        
            //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
            $row = $stmt->fetch(PDO::FETCH_ASSOC);
            
            $this->nom = $row['nom'];
            $this->longitud = number_format($row['longitud'], 5);
            $this->latitud = number_format($row['latitud'], 5);
            $this->observacions = $row['observacions'];
            $this->poblacio = $row['poblacio'];
            $this->codiPostal = $row['codiPostal'];
            $this->esBergueda = $row['bergueda']=='1' ? true : false;
            
        }

            //Funció que retorna un array amb les linies d'una parada
            public function obteLiniesParada(){ //Rep un objecte Parada
                
                 $query = "SELECT A.codiLinia FROM EfectuaParada A JOIN Linia B ON A.codiLinia = B.codi WHERE A.idParada=$this->id ORDER BY B.sigles";
                $stmt = $this->conn->prepare($query);                
                $stmt->execute();
                $arrayLinies=array();
                while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                    $linia = new Linia($this->conn); //Crea un objecte Línia per a cada resposta
                    $linia->codi = $row["codiLinia"];
                    $linia->obteDades(); 
                    array_push($arrayLinies, $linia);
                }
                return $arrayLinies;
                
            }

            public function obteProperesSortides() {
                
                    //PER LES LÍNIES
                    //Preparem l'ordre SQL que inclogui els codis de les linies rebudes
                $grupLinies=$this->obteLiniesParada();
                $ordreLinies="(";
                $primerElement=true;        
                foreach ($grupLinies as $linia) {

                    if (!$primerElement) {
                        $ordreLinies.=" OR ";
                    }          
                    
                    $ordreLinies.="B.codiLinia = ";
                    $ordreLinies.=$linia->codi;
                    $primerElement=false;
                }
                $ordreLinies.=")";

                        //DATA EN FORMAT ANGLÈS
                $data = date("Y-m-d");
                $diaSetmana = diaSetmana($this->conn, $data); //Cridem funció general des de funcions.php
                
                $hora=date("H:i"); //CANVIAR PER HORA REAL
                $hora = date("H:i", strtotime("now - 15 minutes")); //Manté les expedicions de 15 minuts passades
               
                $query = "SELECT A.idExpedicio FROM Horari A JOIN Expedicio B ON A.idExpedicio = B.id
                WHERE 
                   A.idParada = $this->id
                   AND NOT A.idParada=B.paradaFinal
                   AND $ordreLinies
                   AND B.$diaSetmana=1
                   AND A.horaPas >= '$hora'
                   AND (B.diaInici<='$data' OR B.diaInici IS NULL) AND (B.diaFinal>='$data' OR B.diaFinal IS NULL)
                   ORDER BY A.horaPas"; //Pel que fa al dia Inici i dia Final tenim en compte les expedicions que són NULL
                //echo $query;
                $stmt = $this->conn->prepare($query);
                $stmt->execute();
        
                $arrayExpedicions=array();
                while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){ //Creem l'array amb les dades de les expedicions
                    $expedicio = new Expedicio($this->conn); //Crea un objecte Expedicio per a cada resposta
                    $expedicio->id = $row["idExpedicio"];
                    $expedicio->obteDades(); 
                    array_push($arrayExpedicions, $expedicio);
                }
                // echo " Comptador: ";
                // echo count($arrayExpedicions);
                // echo $arrayExpedicions[0]->paradaInici->nom;
                return $arrayExpedicions; //Podem accedir als atributs dels objectes Expedicio
            }


            public function obteOrdre($linia) {
                $query = "SELECT B.ordre FROM Parada A JOIN EfectuaParada B ON A.id=B.idParada WHERE B.codiLinia=$linia->codi AND A.id=$this->id";   
                $stmt = $this->conn->prepare($query);
                $stmt->execute();
                $resposta = $stmt->fetch(PDO::FETCH_ASSOC);   
                return $resposta["ordre"];
            }

            // retorna Array amb els avisos de la linia
            public function obteAvisos(){
                $data=date("Y-m-d"); //Agafa data del dia d'avui, només carreguem els avisos que no han caducat (<dataFinal)            
                $query = "SELECT * FROM Avis A            
                        WHERE
                            A.idObjecte = $this->id
                        AND A.tipusObjecte = 'parada'
                        AND (A.dataFinal>='$data' OR A.dataFinal IS NULL)
                        ORDER BY A.dataFinal
                        ";
                $stmt = $this->conn->prepare( $query );
                $stmt->execute();
                $arrayAvisos=array();

                //Desmuntem les dades rebudes i les assignem a les diferents caselles de l'array
                while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                    $avis = new Avis($this->conn); //Crea un objecte Avis per a cada resposta
                    $avis->id = $row["id"];
                    $avis->obteDadesSenseObjecte(); 
                    array_push($arrayAvisos, $avis);
                }
                
                if (count($arrayAvisos)>0) {
                    return $arrayAvisos; 
                } else null;
                           
            }

}
?>