<?php
// Capçaleres per tal que funcioni l'API
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Headers: access");
header("Access-Control-Allow-Methods: GET");
header("Access-Control-Allow-Credentials: true");
header('Content-Type: application/json');
 
// Agafa l'objecte que genera l'API i les dades de la BD
include_once '../config/database.php';
include_once '../objects/parada.php';
 
// Prepara la connexió a la base de dades
$database = new Database();
$db = $database->getConnection();
 
// crea un nou objecte
$parada = new Parada($db);
 
// Agafa l'ID de la parada i els resultats sol·licitats a partir del rebut a través del GET (.php?id=...)
$parada->id = isset($_GET['id']) ? $_GET['id'] : die();
$limitResultats = isset($_GET['limitResultats']) ? $_GET['limitResultats'] : 4;
 
// Crida la funció de l'objecte
$parada->obteDades();

if($parada->nom!=null){


    //Obtenim dades de la parada concreta
    $resposta=array();
    // Fem una array per emmagatzemar les dades rebudes
    $parada_item = array(
        "id" =>  $parada->id,
        "nom" => $parada->nom,
        "latitud" => $parada->latitud,
        "longitud" => $parada->longitud,
        "observacions" => $parada->observacions,
        "codiPostal" => $parada->codiPostal,
        "poblacio" => $parada->poblacio,
        "esBergueda" => $parada->esBergueda
 
    );
    
    //Posem l'array generada dins d'un altre array
    $resposta["info_parada"] = $parada_item;


    //Obtenim linies de la parada
    $liniesParada=$parada->obteLiniesParada();

    $arrayLinies=array();
        foreach ($liniesParada as $linia) { 
            
            $linia_item = array (
                "codi" => $linia->codi,
                "nom" => $linia->nom,
                "sigles" => $linia->sigles,
                "color" => $linia->color,
                "transport_demanda" => $linia->transport_demanda     
            );
            
            array_push($arrayLinies, $linia_item);
        }
        $resposta["linies"]=$arrayLinies;


    //Obtenim dades de properes sortides
    $expedicions=$parada->obteProperesSortides();
    $resposta["properesSortides"]=generaArrayProperesSortides($expedicions, $parada, $limitResultats);
    //echo count($expedicions);

    $resposta["avisos"]=$parada->obteAvisos();
    

    // Si ha funcionat bé, enviarem el codi 200 (OK) i codifiquem la sortida en JSON
    http_response_code(200);
    echo json_encode($resposta);
}
 
else{
    // Si no ha anat bé retorna el codi 404 i un missatge d'error
    http_response_code(404);
    echo json_encode(array("message" => "No existeix aquesta parada."));
}

function generaArrayProperesSortides($expedicions, $parada, $limitResultats){
    $arrayProperesSortides=array();
    //PREPAREM ARRAY AMB EXPEDICIONS
    if (count($expedicions)>0) {
        $i=0;
        foreach ($expedicions as $expedicio) {
           
            if ($i<$limitResultats) { //Limita als resultats als que es demanen              
                    $arrayExpedicio = array (
                        "id" => $expedicio->id,
                        "linia" => array (
                            "codi" => $expedicio->linia->codi,
                            "nom" => $expedicio->linia->nom,
                            "sigles" => $expedicio->linia->sigles,
                            "color" => $expedicio->linia->color 
                        ),
                        "direccio"  => $expedicio->paradaFinal->nom,
                        "horaPas" => $expedicio->trobaHoraPas($parada)                    
                    );
                    array_push($arrayProperesSortides, $arrayExpedicio);
                    $i++;                
            }
        }
        
        if (count($arrayProperesSortides)>0) {
            $resposta=array(
                "data" => date("d-m-Y"),
                "hora" => date("H:i"),
                "expedicions"=>$arrayProperesSortides,
                "status" => "OK"
            );
        } else {
            $resposta=array(
                "data" => date("d-m-Y"),
                "hora" => date("H:i"),
                "status" => "ERROR",
                "missatge" => "No hi ha expedicions properes en el dia d'avui"
            );
        }
        return $resposta;
    } else return array (
        "data" => date("d-m-Y"),
        "hora" => date("H:i"),
        "status" => "ERROR",
        "missatge" => "No s'han trobat expedicions properes en el dia d'avui");
}
?>