import React, {Component} from 'react';
import axios from 'axios';
import DadesItinerari from './resultats/DadesItinerari';
import {Container, Row, Col, Alert} from 'react-bootstrap';
import AcordioResultats from './resultats/AcordioResultats';
import MapaSuperior from './resultats/MapaSuperior';
import Carregant from './estructura/Carregant';
import Cercador from './Cercador';
import CercadorWidget from './estructura/CercadorWidget';

class Resultats extends Component {

    state = {
        resposta: {
            dadesItinerari: {
                origen: {
                    nom:"A"
                },
                destinacio: {
                    nom:"B"
                }
            }
        },
        itinerariVisible:0,
        generaInterficie: false,
        error: false,
        missatgeAxios: null
      }

      constructor(){
        super()
        this.handleSetItinerariVisible=this.handleSetItinerariVisible.bind(this)
        this.obteDades=this.obteDades.bind(this)
        this.setTitol=this.setTitol.bind(this)
    }

    handleSetItinerariVisible(index){
        //event.preventDefault();
        // const idItinerari = event.target.getAttribute("index");
        const idItinerari = index;

        this.setState({ itinerariVisible: idItinerari});
    }

    setTitol() {
        document.title = "Itinerari entre " + this.state.resposta.dadesItinerari.origen.nom +" i " + this.state.resposta.dadesItinerari.destinacio.nom + " - Mobilitat Berguedà" ;
    }     

    obteDades() {
      const origen=this.props.itinerari.origen;
      const destinacio=this.props.itinerari.destinacio;
      const data=this.props.itinerari.data;
      const hora=this.props.itinerari.hora;
      const baseURL = `https://api.mobilitat.ricardcodina.cat/itinerari/generaItinerari.php?origen=` + origen + `&destinacio=` + destinacio + `&data=` + data + `&hora=` + hora;
  
      //origen=Biblioteca de Gironella&destinacio=Dickens, Berga&data=09-11-2019&hora=10:25
      axios.get(baseURL)
        .then(res => {
          const resposta = res.data;
          this.setState({ resposta, generaInterficie: true });
          this.setTitol();
        }).catch (error => this.setState({error:true, missatgeAxios:error.response.data.missatge})) //si surt ERROR 40X
    }
  
    componentDidMount() {
      this.obteDades();        
    }

    componentDidUpdate(prevProps) { //Si es modifica el recorregut
      if (prevProps.itinerari !== this.props.itinerari) {
        this.obteDades();
      }
    }

    render() {



  
        //this.setItinerariVisible(2);
        return (
          <div>
              
              {!this.state.generaInterficie&&!this.state.error && //Si no ha carregat les dades
                    <Carregant/>
                }
              {this.state.generaInterficie&&(this.state.resposta.itineraris.status=="OK"||this.state.resposta.itineraris.status=="TRANSBORD") && //Genera la interficie quan ha rebut les dades
                   <div>
                    <MapaSuperior
                       dades={this.state.resposta.dadesItinerari}
                      itinerari={this.state.resposta.itineraris.opcions[this.state.itinerariVisible]}
                      itinerariAParadaOrigen={this.state.resposta.itinerariAParadaOrigen.routes[0]}
                      itinerariAParadaDestinacio={this.state.resposta.itinerariDesParadaDestinacio.routes[0]}/>
                    <Container><DadesItinerari dades={this.state.resposta.dadesItinerari}/>
                    <span className="maBoto modificaItinerariLink"><CercadorWidget canviOrigen={this.props.canviOrigen} canviDestinacio={this.props.canviDestinacio} canviHora={this.props.canviHora} canviData={this.props.canviData} itinerari={this.props.itinerari} assignarItinerari={this.props.assignarItinerari} titol="Modifica itinerari"/></span>
                    <AcordioResultats 
                        itinerariAParadaOrigen={this.state.resposta.itinerariAParadaOrigen}
                        itinerariAParadaDestinacio={this.state.resposta.itinerariDesParadaDestinacio}
                        dades={this.state.resposta.dadesItinerari}
                        itineraris={this.state.resposta.itineraris.opcions}
                        itinerariVisible={this.state.itinerariVisible}
                        setItinerariVisible={this.handleSetItinerariVisible}
                         />
                    </Container>
                       
                   </div>
                }

                  {this.state.generaInterficie&&this.state.resposta.itineraris.status=="ERROR1" && //Genera la interficie quan ha rebut les dades
                   <div>
                    <Container className="modulInferior"><DadesItinerari dades={this.state.resposta.dadesItinerari}/>
                
                    <p>No s'han trobat itineraris per la ruta i dies seleccionats.</p>
                    
                    </Container>
                       
                   </div>
                }

                   {this.state.generaInterficie&&(!this.state.resposta.dadesItinerari.paradaOrigen.id||!this.state.resposta.dadesItinerari.paradaDestinacio.id) && //Genera la interficie quan ha rebut les dades
                   <div>
                    <Container className="modulInferior"><DadesItinerari dades={this.state.resposta.dadesItinerari}/>
                
                    <p>No es pot generar una ruta entre els dos punts escollits</p>
                    
                    </Container>
                       
                   </div>
                }

                  {!this.state.generaInterficie&&this.state.error && //Genera la interficie quan ha rebut les dades
                   <div className="modulInferior">
                    <Container >               
                            <Alert variant="danger">{this.state.missatgeAxios}</Alert>
                            
                            <h3>Redefineix la teva cerca</h3>
                            <Cercador canviOrigen={this.props.canviOrigen} canviDestinacio={this.props.canviDestinacio} canviHora={this.props.canviHora} canviData={this.props.canviData} itinerari={this.props.itinerariBuscar} assignarItinerari={this.props.assignarItinerari}/>
                    </Container>
                       
                   </div>
                }
            <br />              
          </div>
        )
      }

}

export default Resultats;