import React, {Component} from 'react';
import {Button} from 'react-bootstrap';
import axios from 'axios';
import {Link} from 'react-router-dom';
import AfegirExpedicio from './expedicions/AfegirExpedicio';
import {FontAwesomeIcon} from '@fortawesome/react-fontawesome';
import EsborrarExpedicio from './expedicions/EsborrarExpedicio';
import EditarExpedicio from './expedicions/EditarExpedicio';

class AdminExpedicions extends Component {
    
    constructor(){
        super()
        this.canviAfegirExpedicio=this.canviAfegirExpedicio.bind(this)
        this.carregaExpedicions=this.carregaExpedicions.bind(this)
        this.obreEsborrarExpedicio=this.obreEsborrarExpedicio.bind(this)
        this.tancaFinestra=this.tancaFinestra.bind(this)
      }

      carregaLinies() {    
        axios.get(`https://api.mobilitat.ricardcodina.cat/linia/`)
          .then(res => {
            if (res.status==200){
                this.setState({linies:res.data.llistat})
            } else this.setState({pantalla:4, missatgeAxios:res.data.missatge})
          }).catch (error => this.setState({pantalla:4, missatgeAxios:error.response.data.missatge})) //si surt ERROR 40X
    
      } 

      carregaExpedicions() {    
        axios.get(`https://api.mobilitat.ricardcodina.cat/expedicio/`)
          .then(res => {
            if (res.status==200){
                this.setState({expedicions:res.data.llistat})
            } else this.setState({pantalla:4, missatgeAxios:res.data.missatge})
          }).catch (error => this.setState({pantalla:4, missatgeAxios:error.response.data.missatge})) //si surt ERROR 40X
    
      } 


    state = {
        afegirExpedicio:false,
        linies: [],
        expedicions: [],
        finestraEsborrar: false,
        finestraEditar: false,
        expedicioEditar: null
    }
    
    canviAfegirExpedicio() {
        this.state.afegirExpedicio ? this.setState({afegirExpedicio:false}) : this.setState({afegirExpedicio:true});
    }

    obreEsborrarExpedicio(expedicio) {
        this.setState({finestraEsborrar:true, expedicioEditar:expedicio});
     }
 
     obreEditarExpedicio(expedicio) {
         this.setState({finestraEditar:true, expedicioEditar:expedicio});
      }
 
 
     tancaFinestra() {
         this.setState({finestraEsborrar:false, finestraEditar:false});
         this.carregaExpedicions();
      }

    colorGris(expedicio){ //COLOR PER LES PARADES FORA DE PERIODES DE CIRCULACIO
        if (!expedicio.circula) {
            return "#d0d0d0";
        } 
    }

    generaTaula(expedicio, index) {
        return (
            <div className="divTableRow liniaResposta noSelect" key={"linia"+index}> 
                <div className="divTableCell">
                    <div style={{backgroundColor: expedicio.linia.color}} className="siglesLinia">
                        {expedicio.linia.sigles}
                    </div>
                </div>                
                <div className="divTableCell">
                <Link to={"/expedicio/"+expedicio.id} style={{color:this.colorGris(expedicio)}}><strong>{expedicio.paradaInici.hora}</strong>: {expedicio.paradaInici.nom} <FontAwesomeIcon icon="arrow-right" color="#000000"/> <strong>{expedicio.paradaFinal.hora}</strong>: {expedicio.paradaFinal.nom}</Link>
                 </div>
                 {/* <div className="divTableCell">
                 <FontAwesomeIcon icon="pencil-alt" className="maBoto" color="#336122" onClick={()=>this.obreEditarExpedicio(expedicio)}/>
                 </div> */}
                 <div className="divTableCell">
                 <FontAwesomeIcon icon="times" className="maBoto" color="#336122" onClick={()=>this.obreEsborrarExpedicio(expedicio)}/>
                 </div>     
        </div>

        )
    }
    
    componentDidMount() {
        this.carregaLinies();
        this.carregaExpedicions();
        if (this.props.match&&this.props.match.params.opcions=="afegir") this.canviAfegirExpedicio(); //Si arribem amb l'opció d'afegir, s'obre el desplegable
      }

      componentDidUpdate(prevProps) { //Si es canvia la url que rebem a traves de match, es torna a cridar al backend.
        if (this.props.match&&prevProps.match.url !== this.props.match.url) {
            this.props.match&&this.props.match.params.opcions=="afegir" ? this.setState({afegirExpedicio:true}) : this.setState({afegirExpedicio:false});
        }
      }

    render() {
        return (
            <div className="modulInferior">
                <h2>Expedicions</h2>
                <Button variant="secondary" onClick={this.canviAfegirExpedicio}>{this.state.afegirExpedicio ? "Tancar" : "Afegir"}</Button>
                {this.state.afegirExpedicio && <AfegirExpedicio token={this.props.token} linies={this.state.linies} refrescaExpedicions={this.carregaExpedicions}/>}
                
                <div className="divTable taulaElements">
                             <div className="divTableBody">
                                {this.state.expedicions.map((expedicio,index)=> this.generaTaula(expedicio, index))}
                            </div>
                        </div>

                {this.state.finestraEsborrar && <EsborrarExpedicio tancar={this.tancaFinestra} expedicio={this.state.expedicioEditar} token={this.props.token}/>}
                {this.state.finestraEditar && <EditarExpedicio tancar={this.tancaFinestra} linies={this.state.linies} expedicio={this.state.expedicioEditar} token={this.props.token}/>}

            </div>
        )
    }
}

export default AdminExpedicions;